@@ -73,7 +73,7 @@ static gint                 gda_data_access_wrapper_get_n_rows      (GdaDataMode
 static gint                 gda_data_access_wrapper_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_access_wrapper_describe_column (GdaDataModel *model, gint col);
 static GdaDataModelAccessFlags gda_data_access_wrapper_get_access_flags(GdaDataModel *model);
-static const GValue      *gda_data_access_wrapper_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_access_wrapper_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 static GdaValueAttribute    gda_data_access_wrapper_get_attributes_at (GdaDataModel *model, gint col, gint row);
 
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GdaDataAccessWrapper *model);
@@ -160,6 +160,7 @@ gda_data_access_wrapper_data_model_init (GdaDataModelClass *iface)
 	iface->i_iter_prev = NULL;
 
 	iface->i_set_value_at = NULL;
+	iface->i_iter_set_value = NULL;
 	iface->i_set_values = NULL;
         iface->i_append_values = NULL;
 	iface->i_append_row = NULL;
@@ -282,20 +283,20 @@ gda_data_access_wrapper_set_property (GObject *object,
 				if (! (model->priv->model_access_flags & GDA_DATA_MODEL_ACCESS_RANDOM)) {
 					model->priv->iter = gda_data_model_create_iter (mod);
 					g_return_if_fail (model->priv->iter);
-					g_signal_connect (G_OBJECT (model->priv->iter), "row_changed",
+					g_signal_connect (G_OBJECT (model->priv->iter), "row-changed",
 							  G_CALLBACK (iter_row_changed_cb), model);
-					g_signal_connect (G_OBJECT (model->priv->iter), "end_of_data",
+					g_signal_connect (G_OBJECT (model->priv->iter), "end-of-data",
 							  G_CALLBACK (iter_end_of_data_cb), model);
 					model->priv->iter_row = -1; /* because model->priv->iter is invalid */
 					model->priv->rows = g_hash_table_new_full (g_direct_hash, g_direct_equal,
 										   NULL, (GDestroyNotify) g_object_unref);
 				}
 				else {
-					g_signal_connect (G_OBJECT (mod), "row_inserted",
+					g_signal_connect (G_OBJECT (mod), "row-inserted",
 							  G_CALLBACK (model_row_inserted_cb), model);
-					g_signal_connect (G_OBJECT (mod), "row_updated",
+					g_signal_connect (G_OBJECT (mod), "row-updated",
 							  G_CALLBACK (model_row_updated_cb), model);
-					g_signal_connect (G_OBJECT (mod), "row_removed",
+					g_signal_connect (G_OBJECT (mod), "row-removed",
 							  G_CALLBACK (model_row_removed_cb), model);
 				}
   
@@ -358,29 +359,6 @@ gda_data_access_wrapper_new (GdaDataModel *model)
 	return GDA_DATA_MODEL (retmodel);
 }
 
-/**
- * gda_data_access_wrapper_row_exists
- * @wrapper: a #GdaDataAccessWrapper objects
- * @row: a row number to test existance
- *
- * Tests if the wrapper model of @wrapper has a row number @row
- *
- * Returns: TRUE if row number @row exists
- */
-gboolean
-gda_data_access_wrapper_row_exists (GdaDataAccessWrapper *wrapper, gint row)
-{
-	g_return_val_if_fail (GDA_IS_DATA_ACCESS_WRAPPER (wrapper), FALSE);
-	
-	if (wrapper->priv->nb_cols == 0)
-		return FALSE;
-
-	if (gda_data_model_get_value_at ((GdaDataModel*) wrapper, 0, row))
-		return TRUE;
-	else
-		return FALSE;
-}
-
 /*
  * GdaDataModel interface implementation
  */
@@ -454,13 +432,22 @@ create_new_row (GdaDataAccessWrapper *model)
 	gint i;
 	GdaRow *row;
 
-	row = gda_row_new ((GdaDataModel *) model, model->priv->nb_cols);
+	row = gda_row_new (model->priv->nb_cols);
 	for (i = 0; i < model->priv->nb_cols; i++) {
 		GdaHolder *holder;
 
 		holder = gda_data_model_iter_get_holder_for_field (model->priv->iter, i);
-		if (holder)
-			gda_row_set_value (row, i, gda_holder_get_value (holder));
+		if (holder) {
+			GValue *dest;
+			const GValue *cvalue = gda_holder_get_value (holder);
+			dest = gda_row_get_value (row, i);
+			if (cvalue) {
+				gda_value_reset_with_type (dest, G_VALUE_TYPE ((GValue *) cvalue));
+				gda_value_set_from_value (dest, cvalue);
+			}
+			else
+				gda_value_set_null (dest);
+		}
 	}
 
 	g_hash_table_insert (model->priv->rows, GINT_TO_POINTER (model->priv->iter_row), row);
@@ -470,7 +457,7 @@ create_new_row (GdaDataAccessWrapper *model)
 }
 
 static const GValue *
-gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	GdaDataAccessWrapper *imodel;
 
@@ -481,13 +468,14 @@ gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
 	g_return_val_if_fail (row >= 0, NULL);
 
 	if (col >= imodel->priv->nb_cols) {
-		g_warning (_("Column %d out of range (0-%d)"), col, imodel->priv->nb_cols);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+			     _("Column %d out of range (0-%d)"), col, imodel->priv->nb_cols - 1);
 		return NULL;
 	}
 
 	if (!imodel->priv->rows)
 		/* imodel->priv->model is a random access model, use it */
-		return gda_data_model_get_value_at (imodel->priv->model, col, row);
+		return gda_data_model_get_value_at (imodel->priv->model, col, row, error);
 	else {
 		GdaRow *gda_row;
 
@@ -502,8 +490,11 @@ gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
 					if (row == imodel->priv->iter_row)
 						return gda_row_get_value (gda_row, col);
 				}
-				else
+				else {
+					g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+						     _("Can't set iterator's position"));
 					return NULL;
+				}
 			}
 				
 			gda_row = NULL;
@@ -528,6 +519,8 @@ gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
 		}
 	}
 
+	g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+		     _("Can't access data"));
 	return NULL;
 }
 
@@ -573,7 +566,7 @@ gda_data_access_wrapper_get_attributes_at (GdaDataModel *model, gint col, gint r
 	imodel = (GdaDataAccessWrapper *) model;
 	g_return_val_if_fail (imodel->priv, 0);
 
-	if (!imodel->priv->rows)
+	if (imodel->priv->model)
 		flags = gda_data_model_get_attributes_at (imodel->priv->model, col, row);
 	flags |= GDA_VALUE_ATTR_NO_MODIF;
 	
