@@ -81,7 +81,7 @@ static gint                 gda_data_model_bdb_get_n_rows      (GdaDataModel *mo
 static gint                 gda_data_model_bdb_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_bdb_describe_column (GdaDataModel *model, gint col);
 static GdaDataModelAccessFlags gda_data_model_bdb_get_access_flags(GdaDataModel *model);
-static const GValue      *gda_data_model_bdb_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_model_bdb_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 static GdaValueAttribute    gda_data_model_bdb_get_attributes_at (GdaDataModel *model, gint col, gint row);
 
 static gboolean             gda_data_model_bdb_set_value_at (GdaDataModel *model, gint col, gint row, const GValue *value, GError **error);
@@ -115,6 +115,7 @@ gda_data_model_bdb_data_model_init (GdaDataModelClass *iface)
         iface->i_iter_prev = NULL;
 
         iface->i_set_value_at = gda_data_model_bdb_set_value_at;
+	iface->i_iter_set_value = NULL;
         iface->i_set_values = gda_data_model_bdb_set_values;
         iface->i_append_values = gda_data_model_bdb_append_values;
         iface->i_append_row = gda_data_model_bdb_append_row;
@@ -158,7 +159,7 @@ gda_data_model_bdb_class_init (GdaDataModelBdbClass *klass)
                                                               G_PARAM_READABLE | G_PARAM_WRITABLE |
                                                               G_PARAM_CONSTRUCT_ONLY));
         g_object_class_install_property (object_class, PROP_DB_NAME,
-                                         g_param_spec_string ("db_name", "Name of the database", NULL, NULL,
+                                         g_param_spec_string ("db-name", "Name of the database", NULL, NULL,
                                                               G_PARAM_READABLE | G_PARAM_WRITABLE |
                                                               G_PARAM_CONSTRUCT_ONLY));
 
@@ -362,7 +363,7 @@ gda_data_model_bdb_set_property (GObject *object,
                         column = gda_column_new ();
                         model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "key");
-			gda_column_set_title (column, "key");
+			gda_column_set_description (column, "key");
 			gda_column_set_g_type (column, GDA_TYPE_BINARY);
 		}
 
@@ -379,7 +380,7 @@ gda_data_model_bdb_set_property (GObject *object,
                         column = gda_column_new ();
                         model->priv->columns = g_slist_append (model->priv->columns , column);
 			gda_column_set_name (column, "data");
-			gda_column_set_title (column, "data");
+			gda_column_set_description (column, "data");
 			gda_column_set_g_type (column, GDA_TYPE_BINARY);
 		}
 		model->priv->n_columns = g_slist_length (model->priv->columns);
@@ -431,7 +432,7 @@ gda_data_model_bdb_new (const gchar *filename, const gchar *db_name)
 
 	g_return_val_if_fail (filename && *filename, NULL);
 
-	model = (GdaDataModel *) g_object_new (GDA_TYPE_DATA_MODEL_BDB, "db_name", db_name, 
+	model = (GdaDataModel *) g_object_new (GDA_TYPE_DATA_MODEL_BDB, "db-name", db_name, 
 					       "filename", filename, NULL);
 
 	return model;
@@ -591,7 +592,7 @@ move_cursor_at (GdaDataModelBdb *model, gint row)
 }
 
 static const GValue *
-gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	GdaDataModelBdb *imodel;
 	DBT key, data;
@@ -604,7 +605,12 @@ gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row)
         g_return_val_if_fail (imodel->priv, NULL);
 
 	if ((col < 0) || (col > imodel->priv->n_columns)) {
-		add_error (imodel, _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, imodel->priv->n_columns - 1);
+		add_error (imodel, tmp);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+			     tmp);
+		g_free (tmp);
 		return NULL;
 	}
 
@@ -616,6 +622,8 @@ gda_data_model_bdb_get_value_at (GdaDataModel *model, gint col, gint row)
 	ret = imodel->priv->dbpc->c_get (imodel->priv->dbpc, &key, &data, DB_CURRENT);
 	if (ret) {
 		add_error (imodel, db_strerror (ret));
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     db_strerror (ret));
 		return NULL;
 	}
 
@@ -701,7 +709,10 @@ gda_data_model_bdb_get_attributes_at (GdaDataModel *model, gint col, gint row)
         g_return_val_if_fail (imodel->priv, 0);
 
 	if ((col < 0) || (col > imodel->priv->n_columns)) {
-		add_error (imodel, _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, imodel->priv->n_columns - 1);
+		add_error (imodel, tmp);
+		g_free (tmp);
 		return 0;
 	}
 
@@ -727,7 +738,12 @@ gda_data_model_bdb_set_value_at (GdaDataModel *model, gint col, gint row, const
         g_return_val_if_fail (imodel->priv, FALSE);
 
 	if ((col < 0) || (col > imodel->priv->n_columns)) {
-		add_error (imodel, _("Column number out of range"));
+		gchar *tmp;
+		tmp = g_strdup_printf (_("Column %d out of range (0-%d)"), col, imodel->priv->n_columns - 1);
+		add_error (imodel, tmp);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+			     tmp);
+		g_free (tmp);
 		return FALSE;
 	}
 
@@ -827,6 +843,8 @@ gda_data_model_bdb_set_values (GdaDataModel *model, gint row, GList *values, GEr
 	ret = imodel->priv->dbpc->c_get (imodel->priv->dbpc, &key, &data, DB_CURRENT);
 	if (ret) {
 		add_error (imodel, db_strerror (ret));
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     db_strerror (ret));
 		return FALSE;
 	}
 
@@ -835,7 +853,7 @@ gda_data_model_bdb_set_values (GdaDataModel *model, gint row, GList *values, GEr
 		return FALSE;
 
 	if (key_modified) {
-		g_set_error (error, 0, 0,
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
 			     _("Key modification is not supported"));
 		return FALSE;
 	}
@@ -888,6 +906,8 @@ gda_data_model_bdb_set_values (GdaDataModel *model, gint row, GList *values, GEr
 		ret = imodel->priv->dbpc->c_put (imodel->priv->dbpc, &key, &data, DB_CURRENT);
 		if (ret) {
 			add_error (imodel, db_strerror (ret));
+			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+				     db_strerror (ret));
 			return FALSE;
 		}
 	}
@@ -929,7 +949,8 @@ gda_data_model_bdb_append_row (GdaDataModel *model, GError **error)
 	ret = imodel->priv->dbp->put (imodel->priv->dbp, NULL, &key, &data, 0);
 	if (ret) {
 		add_error (imodel, db_strerror (ret));
-		g_print ("ERR1\n");
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     db_strerror (ret));
 		return -1;
 	}
 
@@ -955,6 +976,8 @@ gda_data_model_bdb_remove_row (GdaDataModel *model, gint row, GError **error)
 	ret = imodel->priv->dbpc->c_del (imodel->priv->dbpc, 0);
 	if (ret) {
 		add_error (imodel, db_strerror (ret));
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+			     db_strerror (ret));
 		return FALSE;
 	}
 
