@@ -65,8 +65,10 @@ struct _GdaDataProxyClass
 	void                 (* sample_size_changed)  (GdaDataProxy *proxy, gint sample_size);
 	void                 (* sample_changed)       (GdaDataProxy *proxy, gint sample_start, gint sample_end);
 
-	gboolean             (* pre_changes_applied)  (GdaDataProxy *proxy, gint row, gint proxied_row);
-	void                 (* post_changes_applied) (GdaDataProxy *proxy, gint row, gint proxied_row);
+	GError              *(* validate_row_changes) (GdaDataProxy *proxy, gint row, gint proxied_row);
+	void                 (* row_changes_applied)  (GdaDataProxy *proxy, gint row, gint proxied_row);
+
+	void                 (* filter_changed)       (GdaDataProxy *proxy);
 };
 
 GType             gda_data_proxy_get_type                 (void) G_GNUC_CONST;
@@ -87,8 +89,6 @@ void              gda_data_proxy_undelete                 (GdaDataProxy *proxy,
 gboolean          gda_data_proxy_row_is_deleted           (GdaDataProxy *proxy, gint proxy_row);
 gboolean          gda_data_proxy_row_is_inserted          (GdaDataProxy *proxy, gint proxy_row);
 
-gint              gda_data_proxy_find_row_from_values     (GdaDataProxy *proxy, GSList *values, 
-						           gint *cols_index);
 gboolean          gda_data_proxy_row_has_changed          (GdaDataProxy *proxy, gint proxy_row);
 gboolean          gda_data_proxy_has_changed              (GdaDataProxy *proxy);
 gint              gda_data_proxy_get_n_new_rows           (GdaDataProxy *proxy);
