@@ -76,24 +76,27 @@ gda_prepare_create_database (const gchar *provider, const gchar *db_name, GError
 
 /**
  * gda_perform_create_database
+ * @provider: the database provider to use, or %NULL if @op has been created using gda_prepare_create_database()
  * @op: a #GdaServerOperation object obtained using gda_prepare_create_database()
  * @error: a place to store en error, or %NULL
  *
- * Creates a new database using the specifications in @op, which must have been obtained using
- * gda_prepare_create_database ()
+ * Creates a new database using the specifications in @op. @op can be obtained using
+ * gda_server_provider_create_operation(), or gda_prepare_create_database().
  *
  * Returns: TRUE if no error occurred and the database has been created
  */
 gboolean
-gda_perform_create_database (GdaServerOperation *op, GError **error)
+gda_perform_create_database (const gchar *provider, GdaServerOperation *op, GError **error)
 {
-	GdaServerProvider *provider;
+	GdaServerProvider *prov;
 
 	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
-
-	provider = g_object_get_data (G_OBJECT (op), "_gda_provider_obj");
-	if (provider) 
-		return gda_server_provider_perform_operation (provider, NULL, op, error);
+	if (provider)
+		prov = gda_config_get_provider (provider, error);
+	else
+		prov = g_object_get_data (G_OBJECT (op), "_gda_provider_obj");
+	if (prov) 
+		return gda_server_provider_perform_operation (prov, NULL, op, error);
 	else {
 		g_warning ("Could not find operation's associated provider, "
 			   "did you use gda_prepare_create_database() ?");
@@ -142,24 +145,27 @@ gda_prepare_drop_database (const gchar *provider, const gchar *db_name, GError *
 
 /**
  * gda_perform_drop_database
+ * @provider: the database provider to use, or %NULL if @op has been created using gda_prepare_drop_database()
  * @op: a #GdaServerOperation object obtained using gda_prepare_drop_database()
  * @error: a place to store en error, or %NULL
  *
- * Destroys an existing database using the specifications in @op,  which must have been obtained using
- * gda_prepare_drop_database ()
+ * Destroys an existing database using the specifications in @op.  @op can be obtained using
+ * gda_server_provider_create_operation(), or gda_prepare_drop_database().
  *
  * Returns: TRUE if no error occurred and the database has been destroyed
  */
 gboolean
-gda_perform_drop_database (GdaServerOperation *op, GError **error)
+gda_perform_drop_database (const gchar *provider, GdaServerOperation *op, GError **error)
 {
-	GdaServerProvider *provider;
+	GdaServerProvider *prov;
 
 	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), FALSE);
-
-	provider = g_object_get_data (G_OBJECT (op), "_gda_provider_obj");
-	if (provider) 
-		return gda_server_provider_perform_operation (provider, NULL, op, error);
+	if (provider)
+		prov = gda_config_get_provider (provider, error);
+	else
+		prov = g_object_get_data (G_OBJECT (op), "_gda_provider_obj");
+	if (prov) 
+		return gda_server_provider_perform_operation (prov, NULL, op, error);
 	else {
 		g_warning ("Could not find operation's associated provider, "
 			   "did you use gda_prepare_drop_database() ?"); 
@@ -441,7 +447,9 @@ gtype_equal (gconstpointer a, gconstpointer b)
  * gda_get_default_handler
  * @for_type: a #GType type
  * 
- * Obtain a pointer to a #GdaDataHandler which can manage #GValue values of type @for_type
+ * Obtain a pointer to a #GdaDataHandler which can manage #GValue values of type @for_type. The returned
+ * data handler will be adapted to use the current locale information (for example dates will be formatted
+ * taking into accoutn the locale).
  *
  * The returned pointer is %NULL if there is no default data handler available for the @for_type data type
  *
@@ -464,7 +472,7 @@ gda_get_default_handler (GType for_type)
                 g_hash_table_insert (hash, (gpointer) GDA_TYPE_BINARY, gda_handler_bin_new ());
                 g_hash_table_insert (hash, (gpointer) GDA_TYPE_BLOB, gda_handler_bin_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_BOOLEAN, gda_handler_boolean_new ());
-                g_hash_table_insert (hash, (gpointer) G_TYPE_DATE, gda_handler_time_new_no_locale ());
+                g_hash_table_insert (hash, (gpointer) G_TYPE_DATE, gda_handler_time_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_DOUBLE, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_INT, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) GDA_TYPE_NUMERIC, gda_handler_numerical_new ());
@@ -472,8 +480,8 @@ gda_get_default_handler (GType for_type)
                 g_hash_table_insert (hash, (gpointer) GDA_TYPE_SHORT, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) GDA_TYPE_USHORT, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_STRING, gda_handler_string_new ());
-                g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIME, gda_handler_time_new_no_locale ());
-                g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIMESTAMP, gda_handler_time_new_no_locale ());
+                g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIME, gda_handler_time_new ());
+                g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIMESTAMP, gda_handler_time_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_CHAR, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_UCHAR, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_ULONG, gda_handler_numerical_new ());
