@@ -29,6 +29,8 @@
 #include "gda-util.h"
 #include <libgda.h>
 #include <libgda/gda-attributes-manager.h>
+#include <libgda/gda-custom-marshal.h>
+#include <libgda/gda-error.h>
 
 /* 
  * Main static functions 
@@ -149,10 +151,10 @@ validate_change_accumulator (GSignalInvocationHint *ihint,
 {
 	GError *error;
 
-	error = g_value_get_pointer (handler_return);
-	g_value_set_pointer (return_accu, error);
+	error = g_value_get_boxed (handler_return);
+	g_value_set_boxed (return_accu, error);
 
-	return error ? FALSE : TRUE; /* stop signal if 'thisvalue' is FALSE */
+	return error ? FALSE : TRUE; /* stop signal if error has been set */
 }
 
 static GError *
@@ -180,22 +182,22 @@ gda_holder_class_init (GdaHolderClass *class)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaHolderClass, source_changed),
                               NULL, NULL,
-                              gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
+                              _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 	gda_holder_signals[CHANGED] =
                 g_signal_new ("changed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaHolderClass, changed),
                               NULL, NULL,
-                              gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
+                              _gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 	gda_holder_signals[ATT_CHANGED] =
                 g_signal_new ("attribute-changed",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GdaHolderClass, att_changed),
                               NULL, NULL,
-                              gda_marshal_VOID__STRING_POINTER, G_TYPE_NONE, 2, 
-			      G_TYPE_STRING, G_TYPE_POINTER);
+                              _gda_marshal_VOID__STRING_VALUE, G_TYPE_NONE, 2, 
+			      G_TYPE_STRING, G_TYPE_VALUE);
 
 	/**
 	 * GdaHolder::before-change:
@@ -214,7 +216,7 @@ gda_holder_class_init (GdaHolderClass *class)
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdaHolderClass, validate_change),
                               validate_change_accumulator, NULL,
-                              gda_marshal_POINTER__POINTER, G_TYPE_POINTER, 1, G_TYPE_POINTER);
+                              _gda_marshal_ERROR__VALUE, GDA_TYPE_ERROR, 1, G_TYPE_VALUE);
 
         class->changed = NULL;
         class->source_changed = NULL;
@@ -689,7 +691,7 @@ gda_holder_get_value (GdaHolder *holder)
 	g_return_val_if_fail (holder->priv, NULL);
 
 	if (!holder->priv->full_bind) {
-		/* return default value is possible */
+		/* return default value if possible */
 		if (holder->priv->default_forced) {
 			g_assert (holder->priv->default_value);
 			if (G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) 
@@ -728,10 +730,9 @@ gda_holder_get_value_str (GdaHolder *holder, GdaDataHandler *dh)
         if (!current_val || gda_value_is_null (current_val))
                 return NULL;
         else {
-                GdaDataHandler *dh;
-
-                dh = gda_get_default_handler (holder->priv->g_type);
-                if (dh)
+                if (!dh)
+			dh = gda_get_default_handler (holder->priv->g_type);
+		if (dh)
                         return gda_data_handler_get_str_from_value (dh, current_val);
                 else
                         return NULL;
@@ -807,7 +808,7 @@ gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *va
 		if (!dh)
 			dh = gda_get_default_handler (holder->priv->g_type);
 		if (dh)
-        	gdaval = gda_data_handler_get_value_from_str (dh, value, holder->priv->g_type);
+			gdaval = gda_data_handler_get_value_from_str (dh, value, holder->priv->g_type);
 
 		if (gdaval)
 			return real_gda_holder_set_value (holder, gdaval, FALSE, error);
@@ -894,13 +895,15 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 	newvalid = TRUE;
 	if (newnull && holder->priv->not_null) {
 		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_NULL_ERROR,
-			     _("Holder does not allow NULL values"));
+			     _("(%s): Holder does not allow NULL values"),
+			     holder->priv->id);
 		newvalid = FALSE;
 		changed = TRUE;
 	}
 	else if (!newnull && (G_VALUE_TYPE (value) != holder->priv->g_type)) {
 		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_TYPE_ERROR,
-			     _("Wrong value type: expected type '%s' when value's type is '%s'"),
+			     _("(%s): Wrong Holder value type, expected type '%s' when value's type is '%s'"),
+			     holder->priv->id,
 			     gda_g_type_to_string (holder->priv->g_type),
 			     gda_g_type_to_string (G_VALUE_TYPE (value)));
 		newvalid = FALSE;
@@ -1019,13 +1022,15 @@ real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value,
 	newvalid = TRUE;
 	if (newnull && holder->priv->not_null) {
 		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_NULL_ERROR,
-			     _("Holder does not allow NULL values"));
+			     _("(%s): Holder does not allow NULL values"),
+			     holder->priv->id);
 		newvalid = FALSE;
 		changed = TRUE;
 	}
 	else if (!newnull && (G_VALUE_TYPE (value) != holder->priv->g_type)) {
 		g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_TYPE_ERROR,
-			     _("Wrong value type: expected type '%s' when value's type is '%s'"),
+			     _("(%s): Wrong value type: expected type '%s' when value's type is '%s'"),
+			     holder->priv->id,
 			     gda_g_type_to_string (holder->priv->g_type),
 			     gda_g_type_to_string (G_VALUE_TYPE (value)));
 		newvalid = FALSE;
@@ -1048,12 +1053,10 @@ real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value,
 		holder->priv->invalid_forced = FALSE;
 		holder->priv->valid = newvalid;
 #ifdef DEBUG_HOLDER		
-		g_print ("Holder is not changed, returning %p\n", holder->priv->value);
+		g_print ("Holder is not changed");
 #endif		
-
 		/* set the changed status */
 		*value_changed = FALSE;
-		return holder->priv->value;
 	}
 	else {
 		*value_changed = TRUE;
@@ -1099,8 +1102,11 @@ real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value,
 	}
 	else {
 		if (holder->priv->value) {
-			value_to_return = holder->priv->value;
-			holder->priv->value = NULL;
+			if (G_IS_VALUE (holder->priv->value))
+				value_to_return = holder->priv->value;
+			else
+				value_to_return = NULL;
+			holder->priv->value = NULL;			
 		}
 
 		if (value) {
@@ -1112,6 +1118,11 @@ real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value,
 		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
 	}
 
+#ifdef DEBUG_HOLDER	
+	g_print ("returning %p, wannabe was %p\n", value_to_return,
+			 value);
+#endif	
+	
 	return value_to_return;
 }
 
@@ -1603,7 +1614,19 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 static void
 full_bind_changed_cb (GdaHolder *alias_of, GdaHolder *holder)
 {
-	g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
+	if (alias_of == holder->priv->simple_bind) {
+		const GValue *cvalue;
+		GError *error = NULL;
+		cvalue = gda_holder_get_value (alias_of);
+		if (! gda_holder_set_value (holder, cvalue, &error)) {
+			g_warning (_("Could not change GdaHolder to match value change in bound GdaHolder: %s"),
+				   error && error->message ? error->message : _("No detail"));
+			if (error)
+				g_error_free (error);
+		}
+	}
+	else
+		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
 }
 
 /**
@@ -1658,6 +1681,8 @@ const GValue *
 gda_holder_get_attribute (GdaHolder *holder, const gchar *attribute)
 {
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
+	/*g_print ("GdaHolder %p ATTR '%s' get => '%s'\n", holder, attribute, 
+	  gda_value_stringify (gda_attributes_manager_get (gda_holder_attributes_manager, holder, attribute))); */
 	return gda_attributes_manager_get (gda_holder_attributes_manager, holder, attribute);
 }
 
@@ -1687,4 +1712,5 @@ gda_holder_set_attribute (GdaHolder *holder, const gchar *attribute, const GValu
 		return;
 
 	gda_attributes_manager_set (gda_holder_attributes_manager, holder, attribute, value);
+	//g_print ("GdaHolder %p ATTR '%s' set to '%s'\n", holder, attribute, gda_value_stringify (value)); 
 }
