@@ -138,7 +138,7 @@ gda_statement_class_init (GdaStatementClass * klass)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaStatementClass, checked),
 			      NULL, NULL,
-			      gda_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE,
+			      _gda_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE,
 			      2, GDA_TYPE_CONNECTION, G_TYPE_BOOLEAN);
 
 	klass->reset = NULL;
@@ -449,24 +449,6 @@ gda_statement_serialize (GdaStatement *stmt)
 	return str;
 }
 
-/**
- * gda_statement_deserialize
- * @str: a string containing a serialized version of a #GdaStatement
- * @error: a place to store errors, or %NULL
- *
- * Creates a new #GdaStatement from a string
- *
- * Returns: a new #GdaStatement object, or %NULL if an error occurred
- */
-GdaStatement *
-gda_statement_deserialize (const gchar *str, GError **error)
-{
-	g_return_val_if_fail (str || *str, NULL);
-	TO_IMPLEMENT;
-
-	return NULL;
-}
-
 static gboolean
 get_params_foreach_func (GdaSqlAnyPart *node, GdaSet **params, GError **error)
 {
@@ -677,7 +659,7 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 	if (value && !gda_value_is_null (value)) {
 		GdaDataHandler *dh;
 		if (context->provider)
-			dh = gda_server_provider_get_data_handler_gtype (context->provider, context->cnc, G_VALUE_TYPE (value));
+			dh = gda_server_provider_get_data_handler_g_type (context->provider, context->cnc, G_VALUE_TYPE (value));
 		else  			
 			dh = gda_get_default_handler (G_VALUE_TYPE (value));
 
