@@ -29,6 +29,7 @@
 #include "gda-holder.h"
 #include "gda-row.h"
 #include "gda-connection.h"
+#include <sql-parser/gda-sql-statement.h>
 
 G_BEGIN_DECLS
 
@@ -51,10 +52,10 @@ gchar      **gda_completion_list_get (GdaConnection *cnc, const gchar *text, gin
  * Param & model utilities
  */
 gboolean     gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...);
-void         gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
+gboolean     gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
 					      const gint *cols, gint nb_cols, const gint *rows, gint nb_rows,
 					      gboolean use_col_ids);
-void         gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *sources);
+gboolean     gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *sources, GError **error);
 
 /* 
  * translate any text to an alphanumerical text 
@@ -65,6 +66,8 @@ gchar       *gda_alphanum_to_text (gchar *text);
 /*
  * Statement computation from meta store 
  */
+GdaSqlExpr  *gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTable *mtable, 
+						     gboolean require_pk, GError **error);
 gboolean     gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gboolean require_pk, 
 					 GdaStatement **insert_stmt, GdaStatement **update_stmt, GdaStatement **delete_stmt, 
 					 GError **error);
@@ -78,6 +81,14 @@ void         gda_dsn_split               (const gchar *string, gchar **out_dsn,
 					  gchar **out_username, gchar **out_password);
 void         gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar **out_provider, 
 					  gchar **out_username, gchar **out_password);
+
+/*
+ * Date and time parsing from ISO 8601 (well, part of it)
+ */
+gboolean     gda_parse_iso8601_date (GDate *gdate, const gchar *value);
+gboolean     gda_parse_iso8601_time (GdaTime *timegda, const gchar *value);
+gboolean     gda_parse_iso8601_timestamp (GdaTimestamp *timestamp, const gchar *value);
+
 G_END_DECLS
 
 #endif
