@@ -22,6 +22,8 @@
 #include <libgda/sql-parser/gda-statement-struct.h>
 #include <libgda/gda-debug-macros.h>
 #include <libgda/gda-connection.h>
+#include <libgda/gda-meta-struct.h>
+#include <libgda/gda-util.h>
 #include <libgda/sql-parser/gda-statement-struct-util.h>
 #include <libgda/sql-parser/gda-statement-struct-unknown.h>
 #include <libgda/sql-parser/gda-statement-struct-trans.h>
@@ -49,7 +51,10 @@ gda_sql_error_quark (void)
 GdaSqlStatementContentsInfo *
 gda_sql_statement_get_contents_infos  (GdaSqlStatementType type) 
 {
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 	static GdaSqlStatementContentsInfo **contents = NULL;
+
+	g_static_mutex_lock (&mutex);
 	if (!contents) {
 		contents = g_new0 (GdaSqlStatementContentsInfo *, GDA_SQL_STATEMENT_NONE);
 
@@ -66,6 +71,8 @@ gda_sql_statement_get_contents_infos  (GdaSqlStatementType type)
 		contents [GDA_SQL_STATEMENT_ROLLBACK_SAVEPOINT] = gda_sql_statement_rollback_savepoint_get_infos ();
 		contents [GDA_SQL_STATEMENT_DELETE_SAVEPOINT] = gda_sql_statement_delete_savepoint_get_infos ();
 	}
+	g_static_mutex_unlock (&mutex);
+
 	return contents[type];
 }
 
@@ -102,15 +109,25 @@ gda_sql_statement_copy (GdaSqlStatement *stmt)
 		return NULL;
 
 	infos = gda_sql_statement_get_contents_infos (stmt->stmt_type);
-	copy = gda_sql_statement_new (stmt->stmt_type);
+	copy = g_new0 (GdaSqlStatement, 1);
+	copy->stmt_type = stmt->stmt_type;
+
 	if (stmt->sql)
 		copy->sql = g_strdup (stmt->sql);
 	if (infos && infos->copy) {
 		copy->contents = infos->copy (stmt->contents);
 		GDA_SQL_ANY_PART (copy->contents)->type = GDA_SQL_ANY_PART (stmt->contents)->type;
 	}
-	else 
+	else if (infos && infos->construct) {
+		copy->contents = infos->construct ();
+		GDA_SQL_ANY_PART (copy->contents)->type = stmt->stmt_type;
+	}
+	else
 		TO_IMPLEMENT;
+	if (stmt->validity_meta_struct) {
+		copy->validity_meta_struct = stmt->validity_meta_struct;
+		g_object_ref (copy->validity_meta_struct);
+	}
 
 	return copy;
 }
@@ -129,6 +146,9 @@ gda_sql_statement_free (GdaSqlStatement *stmt)
 		else 
 			TO_IMPLEMENT;
 	}
+	if (stmt->validity_meta_struct)
+		g_object_unref (stmt->validity_meta_struct);
+
 	g_free (stmt);
 }
 
@@ -223,64 +243,188 @@ gda_sql_statement_serialize (GdaSqlStatement *stmt)
 /*
  * Check with dict data structure
  */
-typedef struct {
-	GdaConnection *cnc;
-} DictCheckData;
+static gboolean foreach_check_clean (GdaSqlAnyPart *node, gpointer data, GError **error);
 
-static gboolean foreach_check_cnc (GdaSqlAnyPart *node, DictCheckData *data, GError **error);
-static gboolean gda_sql_expr_check_cnc (GdaSqlExpr *expr, DictCheckData *data, GError **error);
-static gboolean gda_sql_field_check_cnc (GdaSqlField *field, DictCheckData *data, GError **error);
-static gboolean gda_sql_table_check_cnc (GdaSqlTable *table, DictCheckData *data, GError **error);
-static gboolean gda_sql_function_check_cnc (GdaSqlFunction *function, DictCheckData *data, GError **error);
-static gboolean gda_sql_select_field_check_cnc (GdaSqlSelectField *field, DictCheckData *data, GError **error);
-static gboolean gda_sql_select_target_check_cnc (GdaSqlSelectTarget *target, DictCheckData *data, GError **error);
+static gboolean foreach_check_validity (GdaSqlAnyPart *node, GdaSqlStatementCheckValidityData *data, GError **error);
+static gboolean gda_sql_expr_check_validity (GdaSqlExpr *expr, GdaSqlStatementCheckValidityData *data, GError **error);
+static gboolean gda_sql_field_check_validity (GdaSqlField *field, GdaSqlStatementCheckValidityData *data, GError **error);
+static gboolean gda_sql_table_check_validity (GdaSqlTable *table, GdaSqlStatementCheckValidityData *data, GError **error);
+static gboolean gda_sql_function_check_validity (GdaSqlFunction *function, GdaSqlStatementCheckValidityData *data, GError **error);
+static gboolean gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCheckValidityData *data, GError **error);
+static gboolean gda_sql_select_target_check_validity (GdaSqlSelectTarget *target, GdaSqlStatementCheckValidityData *data, GError **error);
 
 /**
- * gda_sql_statement_check_connection
+ * gda_sql_statement_check_validity
  * @stmt: a #GdaSqlStatement pointer
  * @cnc: a #GdaConnection object, or %NULL
  * @error: a place to store errors, or %NULL
  *
  * If @cnc is not %NULL, then checks that all the database objects referenced in the statement actually
  * exist in the connection's database (for example the table being updated in a UPDATE statement must exist in the
- * connection's database for the check to succeed).
+ * connection's database for the check to succeed). This method fills the @stmt-&gt;validity_meta_struct attribute.
  *
- * If @cnc is %NULL, then remove any information from a previous call to this method stored in @stmt.
+ * If @cnc is %NULL, then remove any information from a previous call to this method stored in @stmt. In this case,
+ * the @stmt-&gt;validity_meta_struct attribute is cleared.
+ *
+ * Also note that some parts of @stmt may be modified: for example leading and trailing spaces in aliases or
+ * objects names will be removed.
  *
  * Returns: TRUE if no error occurred
  */
 gboolean
-gda_sql_statement_check_connection (GdaSqlStatement *stmt, GdaConnection *cnc, GError **error)
+gda_sql_statement_check_validity (GdaSqlStatement *stmt, GdaConnection *cnc, GError **error)
 {
-	DictCheckData data;
-
 	g_return_val_if_fail (stmt, FALSE);
 	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), FALSE);
 
-	data.cnc = cnc;
+	/* check the structure first */
+	if (!gda_sql_statement_check_structure (stmt, error))
+		return FALSE;
 
-	return gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
-					 (GdaSqlForeachFunc) foreach_check_cnc, &data, error);
+	/* clear any previous setting */
+	gda_sql_statement_check_clean (stmt);
+
+	if (cnc) {
+		GdaSqlStatementCheckValidityData data;
+		gboolean retval;
+
+		/* prepare data */
+		data.cnc = cnc;
+		data.store = gda_connection_get_meta_store (cnc);
+		data.mstruct = gda_meta_struct_new (data.store, GDA_META_STRUCT_FEATURE_NONE);
+		
+		/* attach the GdaMetaStruct to @stmt */
+		stmt->validity_meta_struct = data.mstruct;
+		retval = gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
+						   (GdaSqlForeachFunc) foreach_check_validity, &data, error);
+		return retval;
+	}
+	else
+		return TRUE;
 }
 
 static gboolean
-foreach_check_cnc (GdaSqlAnyPart *node, DictCheckData *data, GError **error)
+foreach_check_validity (GdaSqlAnyPart *node, GdaSqlStatementCheckValidityData *data, GError **error)
 {
 	if (!node) return TRUE;
 
 	switch (node->type) {
-	case GDA_SQL_ANY_EXPR:
-		return gda_sql_expr_check_cnc ((GdaSqlExpr*) node, data, error);
-	case GDA_SQL_ANY_SQL_FIELD:
-		return gda_sql_field_check_cnc ((GdaSqlField*) node, data, error);
-	case GDA_SQL_ANY_SQL_TABLE:
-		return gda_sql_table_check_cnc ((GdaSqlTable*) node, data, error);
-	case GDA_SQL_ANY_SQL_FUNCTION:
-		return gda_sql_function_check_cnc ((GdaSqlFunction*) node, data, error);
-	case GDA_SQL_ANY_SQL_SELECT_FIELD:
-		return gda_sql_select_field_check_cnc ((GdaSqlSelectField*) node, data, error);
-	case GDA_SQL_ANY_SQL_SELECT_TARGET:
-		return gda_sql_select_target_check_cnc ((GdaSqlSelectTarget*) node, data, error);
+	case GDA_SQL_ANY_STMT_SELECT:
+        case GDA_SQL_ANY_STMT_INSERT:
+        case GDA_SQL_ANY_STMT_UPDATE:
+        case GDA_SQL_ANY_STMT_DELETE:
+        case GDA_SQL_ANY_STMT_COMPOUND:
+        case GDA_SQL_ANY_STMT_BEGIN:
+        case GDA_SQL_ANY_STMT_ROLLBACK:
+        case GDA_SQL_ANY_STMT_COMMIT:
+        case GDA_SQL_ANY_STMT_SAVEPOINT:
+        case GDA_SQL_ANY_STMT_ROLLBACK_SAVEPOINT:
+        case GDA_SQL_ANY_STMT_DELETE_SAVEPOINT:
+        case GDA_SQL_ANY_STMT_UNKNOWN: {
+		GdaSqlStatementContentsInfo *cinfo;
+		cinfo = gda_sql_statement_get_contents_infos (node->type);
+		if (cinfo->check_validity_func)
+			return cinfo->check_validity_func (node, data, error);
+		break;
+	}
+	case GDA_SQL_ANY_EXPR: {
+		GdaSqlExpr *expr = (GdaSqlExpr *) node;
+		if (expr->cast_as) {
+			g_strchomp (expr->cast_as);
+			if (! *(expr->cast_as)) {
+				g_free (expr->cast_as);
+				expr->cast_as = NULL;
+			}
+		}
+		return gda_sql_expr_check_validity (expr, data, error);
+	}
+	case GDA_SQL_ANY_SQL_FIELD: {
+		GdaSqlField *field = (GdaSqlField*) node;
+		if (field->field_name) {
+			g_strchomp (field->field_name);
+			if (! *(field->field_name)) {
+				g_free (field->field_name);
+				field->field_name = NULL;
+			}
+		}
+		return gda_sql_field_check_validity (field, data, error);
+	}
+	case GDA_SQL_ANY_SQL_TABLE: {
+		GdaSqlTable *table = (GdaSqlTable*) node;
+		if (table->table_name) {
+			g_strchomp (table->table_name);
+			if (! *(table->table_name)) {
+				g_free (table->table_name);
+				table->table_name = NULL;
+			}
+		}
+		return gda_sql_table_check_validity (table, data, error);
+	}
+	case GDA_SQL_ANY_SQL_FUNCTION: {
+		GdaSqlFunction *function = (GdaSqlFunction*) node;
+		if (function->function_name) {
+			g_strchomp (function->function_name);
+			if (! *(function->function_name)) {
+				g_free (function->function_name);
+				function->function_name = NULL;
+			}
+		}
+		return gda_sql_function_check_validity (function, data, error);
+	}
+	case GDA_SQL_ANY_SQL_SELECT_FIELD: {
+		GdaSqlSelectField *field = (GdaSqlSelectField*) node;
+		if (field->as) {
+			g_strchomp (field->as);
+			if (! *(field->as)) {
+				g_free (field->as);
+				field->as = NULL;
+			}
+		}
+
+		if (field->expr && field->expr->value && (G_VALUE_TYPE (field->expr->value) == G_TYPE_STRING)) {
+			g_free (field->field_name);
+			g_free (field->table_name);
+			_split_identifier_string (g_value_dup_string (field->expr->value), &(field->table_name),
+						  &(field->field_name));
+		}
+		if (field->table_name) {
+			g_strchomp (field->table_name);
+			if (! *(field->table_name)) {
+				g_free (field->table_name);
+				field->table_name = NULL;
+			}
+		}
+		if (field->field_name) {
+			g_strchomp (field->field_name);
+			if (! *(field->field_name)) {
+				g_free (field->field_name);
+				field->field_name = NULL;
+			}
+		}
+		return gda_sql_select_field_check_validity (field, data, error);
+	}
+	case GDA_SQL_ANY_SQL_SELECT_TARGET: {
+		GdaSqlSelectTarget *target = (GdaSqlSelectTarget*) node;
+		if (target->as) {
+			g_strchomp (target->as);
+			if (! *(target->as)) {
+				g_free (target->as);
+				target->as = NULL;
+			}
+		}
+		if (target->expr && target->expr->value && (G_VALUE_TYPE (target->expr->value) == G_TYPE_STRING)) {
+			g_free (target->table_name);
+			target->table_name = g_value_dup_string (target->expr->value);
+		}
+		if (target->table_name) {
+			g_strchomp (target->table_name);
+			if (! *(target->table_name)) {
+				g_free (target->table_name);
+				target->table_name = NULL;
+			}
+		}
+		return gda_sql_select_target_check_validity (target, data, error);
+	}
 	default:
 		break;
 	}
@@ -288,23 +432,31 @@ foreach_check_cnc (GdaSqlAnyPart *node, DictCheckData *data, GError **error)
 }
 
 static gboolean
-gda_sql_expr_check_cnc (GdaSqlExpr *expr, DictCheckData *data, GError **error)
+gda_sql_expr_check_validity (GdaSqlExpr *expr, GdaSqlStatementCheckValidityData *data, GError **error)
 {
 	if (!expr) return TRUE;
 	if (!expr->param_spec) return TRUE;
 	gda_sql_expr_check_clean (expr);
 
 	/* 2 checks to do here:
-	 *  - try to find the GdaDictType from expr->param_spec->type
+	 *  - try to find the data type from expr->param_spec->type using data->mstruct (not yet possible)
 	 *  - if expr->param_spec->type is NULL, then try to identify it (and expr->param_spec->g_type)
-	 *    using @expr context
+	 *    using @expr context, set expr->param_spec->validity_meta_dict.
 	 */
         TO_IMPLEMENT;
-        return FALSE;
+        return TRUE;
+}
+
+void
+gda_sql_expr_check_clean (GdaSqlExpr *expr)
+{
+	if (!expr) return;
+	if (expr->param_spec && expr->param_spec->validity_meta_dict)
+		TO_IMPLEMENT;
 }
 
 static gboolean
-gda_sql_field_check_cnc (GdaSqlField *field, DictCheckData *data, GError **error)
+gda_sql_field_check_validity (GdaSqlField *field, GdaSqlStatementCheckValidityData *data, GError **error)
 {
 	GdaSqlAnyPart *any;
 	GdaSqlTable *stable;
@@ -331,144 +483,314 @@ gda_sql_field_check_cnc (GdaSqlField *field, DictCheckData *data, GError **error
 		g_assert_not_reached ();
 
 	if (!stable) {
-		if (any->type == GDA_SQL_ANY_STMT_INSERT)
-			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-				     _("Missing table in INSERT statement"));
-		else
-			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-				     _("Missing table in UPDATE statement"));
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+			     _("Missing table in statement"));
 		return FALSE;
 	}
-	if (!stable->full_table_name) {
-		if (! gda_sql_table_check_cnc (stable, data, error))
+	if (!stable->validity_meta_object) {
+		if (! gda_sql_table_check_validity (stable, data, error))
 			return FALSE;
-		g_assert (stable->full_table_name);
-
-		GdaDataModel *model;
-		GValue *v1, *v2;
-		g_value_set_string (v1 = gda_value_new (G_TYPE_STRING), stable->table_name);
-		g_value_set_string (v2 = gda_value_new (G_TYPE_STRING), field->field_name);
-		model = gda_connection_get_meta_store_data (data->cnc, GDA_CONNECTION_META_FIELDS, error, 2,
-							    "name", v1, "field_name", v2);
-		if (!model)
-			return FALSE;
-		if (gda_data_model_get_n_rows (model) == 0) {
-			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_DICT_ELEMENT_MISSING_ERROR,
-				     _("Field %s.%s not found"), stable->table_name, field->field_name);
-			g_object_unref (model);
-			return FALSE;
-		}
-		g_object_unref (model);
-		return TRUE;
 	}
 
+	g_assert (stable->validity_meta_object);
+	GdaMetaTableColumn *tcol;
+	GValue value;
+	memset (&value, 0, sizeof (GValue));
+	g_value_set_string (g_value_init (&value, G_TYPE_STRING), field->field_name);
+	tcol = gda_meta_struct_get_table_column (data->mstruct, 
+						 GDA_META_TABLE (stable->validity_meta_object), 
+						 &value);
+	g_value_unset (&value);
+	field->validity_meta_table_column = tcol;
+	if (!tcol) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+			     _("Column '%s' not found"), field->field_name);
+		return FALSE;
+	}
+	
         return TRUE;
 }
 
-void
-gda_sql_expr_check_clean (GdaSqlExpr *expr)
-{
-	if (!expr) return;
-	if (expr->param_spec && expr->param_spec->dict_type)
-		TO_IMPLEMENT;
-}
-
 void 
 gda_sql_field_check_clean (GdaSqlField *field)
 {
 	if (!field) return;
-	/* nothing to do */
+	field->validity_meta_table_column = NULL;
+}
+
+/* For GdaSqlSelectTarget, GdaSqlSelectField and GdaSqlTable */
+static GdaMetaDbObject *
+find_table_or_view (GdaSqlAnyPart *part, GdaSqlStatementCheckValidityData *data, const gchar *name, GError **error)
+{
+	GdaMetaDbObject *dbo;
+	GValue value;
+	GError *lerror = NULL;
+	memset (&value, 0, sizeof (GValue));
+
+	/* use @name as the table or view's real name */
+	g_value_set_string (g_value_init (&value, G_TYPE_STRING), name);
+	dbo = gda_meta_struct_complement (data->mstruct, GDA_META_DB_UNKNOWN,
+					  NULL, NULL, &value, &lerror);
+	g_value_unset (&value);
+	if (!dbo) {
+		/* use @name as a table alias in the statement */
+		GdaSqlAnyPart *any;
+
+		for (any = part->parent; any && any->parent; any = any->parent);
+		if (!any) 
+			g_set_error (&lerror, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+				     _("GdaSqlSelectField is not part of a SELECT statement"));
+		else {
+			switch (any->type) {
+			case GDA_SQL_ANY_STMT_SELECT: {
+				GdaSqlStatementSelect *select = (GdaSqlStatementSelect*) any;
+				if (select->from) {
+					GSList *targets;
+					for (targets = select->from->targets; targets; targets = targets->next) {
+						GdaSqlSelectTarget *target = (GdaSqlSelectTarget*) targets->data;
+						if (!target->as)
+							continue;
+						g_value_set_string (g_value_init (&value, G_TYPE_STRING), 
+								    target->table_name);
+						dbo = gda_meta_struct_complement (data->mstruct, 
+										  GDA_META_DB_UNKNOWN,
+										  NULL, NULL, &value, NULL);
+						g_value_unset (&value);
+						if (dbo)
+							break;
+					}
+				}
+				break;
+			}
+			case GDA_SQL_ANY_STMT_INSERT:
+				TO_IMPLEMENT;
+				break;
+			case GDA_SQL_ANY_STMT_UPDATE:
+				TO_IMPLEMENT;
+				break;
+			case GDA_SQL_ANY_STMT_DELETE:
+				TO_IMPLEMENT;
+				break;
+			default:
+				g_assert_not_reached ();
+				break;
+			}
+		}
+	}
+	if (dbo) {
+		if (lerror)
+			g_error_free (lerror);
+	}
+	else {
+		if (lerror)
+			g_propagate_error (error, lerror);
+	}
+	return dbo;
 }
 
 static gboolean
-gda_sql_table_check_cnc (GdaSqlTable *table, DictCheckData *data, GError **error)
+gda_sql_table_check_validity (GdaSqlTable *table, GdaSqlStatementCheckValidityData *data, GError **error)
 {
-	if (!table) return TRUE;
+	GdaMetaDbObject *dbo;
 
-        TO_IMPLEMENT;
-        return FALSE;
+	if (!table) return TRUE;
+	gda_sql_table_check_clean (table);
+
+	if (!table->table_name) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+			     _("Missing table name in statement"));
+		return FALSE;
+	}
+
+	dbo = find_table_or_view ((GdaSqlAnyPart*) table, data, table->table_name, error);
+	if (dbo && ((dbo->obj_type != GDA_META_DB_TABLE) ||
+		    (dbo->obj_type != GDA_META_DB_VIEW))) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+			     _("Table '%s' not found"), table->table_name);
+		return FALSE;
+	}
+	table->validity_meta_object = dbo;
+
+	return table->validity_meta_object ? TRUE : FALSE;
 }
 
 void 
 gda_sql_table_check_clean (GdaSqlTable *table)
 {
 	if (!table) return;
-	if (table->full_table_name) {
-		g_free (table->full_table_name);
-		table->full_table_name = NULL;
-	}
+	table->validity_meta_object = NULL;
 }
 
 static gboolean
-gda_sql_function_check_cnc (GdaSqlFunction *function, DictCheckData *data, GError **error)
+gda_sql_function_check_validity (GdaSqlFunction *function, GdaSqlStatementCheckValidityData *data, GError **error)
 {
 	if (!function) return TRUE;
 
         TO_IMPLEMENT;
-        return FALSE;
+        return TRUE;
 }
 
 void
 gda_sql_function_check_clean (GdaSqlFunction *function)
 {
 	if (!function) return;
-	if (function->full_function_name) {
-		g_free (function->full_function_name);
-		function->full_function_name = NULL;
-	}
+	function->validity_meta_function = NULL;
 }
 
 static gboolean
-gda_sql_select_field_check_cnc (GdaSqlSelectField *field, DictCheckData *data, GError **error)
+gda_sql_select_field_check_validity (GdaSqlSelectField *field, GdaSqlStatementCheckValidityData *data, GError **error)
 {
-	if (!field) return TRUE;
+	GValue value;
+	GdaMetaDbObject *dbo = NULL;
+	gboolean starred_field = FALSE;
 
-	TO_IMPLEMENT;
-	return FALSE;
+	if (!field) return TRUE;
+	gda_sql_select_field_check_clean (field);
+
+	if (!field->field_name) 
+		/* field is not a table.field */
+		return TRUE;
+
+	memset (&value, 0, sizeof (GValue));
+	if (gda_identifier_equal (field->field_name, "*"))
+		starred_field = TRUE;
+
+	if (!field->table_name) {
+		/* go through all the SELECT's targets to see if there is a table with the corresponding field */
+		GdaSqlAnyPart *any;
+		GSList *targets;
+		GdaMetaTableColumn *tcol = NULL;
+
+		for (any = GDA_SQL_ANY_PART(field)->parent; 
+		     any && (any->type != GDA_SQL_ANY_STMT_SELECT);
+		     any = any->parent);
+		if (!any) {
+			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+				     _("GdaSqlSelectField is not part of a SELECT statement"));
+			return FALSE;
+		}
+
+		for (targets = ((GdaSqlStatementSelect *)any)->from->targets; targets; targets = targets->next) {
+			GdaSqlSelectTarget *target = (GdaSqlSelectTarget *) targets->data;
+			if (!target->validity_meta_object && 
+			    !gda_sql_select_target_check_validity (target, data, error))
+				return FALSE;
+
+			g_value_set_string (g_value_init (&value, G_TYPE_STRING), field->field_name);
+			tcol = gda_meta_struct_get_table_column (data->mstruct, 
+								 GDA_META_TABLE (target->validity_meta_object),
+								 &value);
+			g_value_unset (&value);
+			if (tcol) {
+				/* found a candidate */
+				if (dbo) {
+					g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+						     _("Could not identify table for field '%s'"), field->field_name);
+					return FALSE;
+				}
+				dbo = target->validity_meta_object;
+			}
+		}
+		if (!dbo) {
+			targets = ((GdaSqlStatementSelect *)any)->from->targets;
+			if (starred_field && targets && !targets->next) 
+				/* only one target => it's the one */
+				dbo = ((GdaSqlSelectTarget*) targets->data)->validity_meta_object;
+			else {
+				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+					     _("Could not identify table for field '%s'"), field->field_name);
+				return FALSE;
+			}
+		}
+		field->validity_meta_object = dbo;
+		field->validity_meta_table_column = tcol;
+	}
+	else {
+		/* table part */
+		dbo = find_table_or_view ((GdaSqlAnyPart*) field, data, field->table_name, error);
+		if (dbo && (dbo->obj_type != GDA_META_DB_TABLE) &&
+		    (dbo->obj_type != GDA_META_DB_VIEW)) {
+			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+				     _("Table '%s' not found"), field->table_name);
+			return FALSE;
+		}
+		field->validity_meta_object = dbo;
+		if (!field->validity_meta_object)
+			return FALSE;
+		
+		/* field part */
+		if (!gda_identifier_equal (field->field_name, "*")) {
+			GdaMetaTableColumn *tcol;
+			g_value_set_string (g_value_init (&value, G_TYPE_STRING), field->field_name);
+			tcol = gda_meta_struct_get_table_column (data->mstruct, 
+								 GDA_META_TABLE (field->validity_meta_object), 
+								 &value);
+			g_value_unset (&value);
+			field->validity_meta_table_column = tcol;
+			if (!tcol) {
+				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+					     _("Column '%s' not found"), field->field_name);
+				return FALSE;
+			}
+		}
+	}
+	return TRUE;
 }
 
 void 
 gda_sql_select_field_check_clean (GdaSqlSelectField *field)
 {
 	if (!field) return;
-	if (field->full_table_name) {
-		g_free (field->full_table_name);
-		field->full_table_name = NULL;
-	}
+	field->validity_meta_object = NULL;
+	field->validity_meta_table_column = NULL;
 }
 
 static gboolean
-gda_sql_select_target_check_cnc (GdaSqlSelectTarget *target, DictCheckData *data, GError **error)
+gda_sql_select_target_check_validity (GdaSqlSelectTarget *target, GdaSqlStatementCheckValidityData *data, GError **error)
 {
-	if (!target) return TRUE;
+	GdaMetaDbObject *dbo;
 
-	TO_IMPLEMENT;
-	return FALSE;
+	if (!target) return TRUE;
+	if (!target->table_name)
+		return TRUE;
+
+	dbo = find_table_or_view ((GdaSqlAnyPart*) target, data, target->table_name, error);
+	if (dbo && (dbo->obj_type != GDA_META_DB_TABLE) &&
+	    (dbo->obj_type != GDA_META_DB_VIEW)) {
+		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_VALIDATION_ERROR,
+			     _("Table '%s' not found"), target->table_name);
+		return FALSE;
+	}
+	target->validity_meta_object = dbo;
+	return target->validity_meta_object ? TRUE : FALSE;
 }
 
 void 
 gda_sql_select_target_check_clean (GdaSqlSelectTarget *target)
 {
 	if (!target) return;
-	if (target->full_table_name) {
-		g_free (target->full_table_name);
-		target->full_table_name = NULL;
-	}
+	target->validity_meta_object = NULL;
 }
 
 
 
-static gboolean foreach_check_clean (GdaSqlAnyPart *node, gpointer data, GError **error);
 /**
  * gda_sql_statement_check_clean
+ * @stmt: a pinter to a #GdaSqlStatement structure
+ *
+ * Cleans any data set by a previous call to gda_sql_statement_check_validity().
  */
 void
 gda_sql_statement_check_clean (GdaSqlStatement *stmt)
 {
 	g_return_if_fail (stmt);
 
-	gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
-				  (GdaSqlForeachFunc) foreach_check_clean, NULL, NULL);
+	if (stmt->validity_meta_struct) {
+		gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
+					  (GdaSqlForeachFunc) foreach_check_clean, NULL, NULL);
+		g_object_unref (stmt->validity_meta_struct);
+		stmt->validity_meta_struct = NULL;
+	}
 }
 
 static gboolean
@@ -609,61 +931,61 @@ gda_sql_any_part_check_structure (GdaSqlAnyPart *node, GError **error)
 				     _("Operation has no operand"));
 			return FALSE;
 		}
-		switch (operation->operator) {
-		case GDA_SQL_OPERATOR_EQ:
-		case GDA_SQL_OPERATOR_IS:
-		case GDA_SQL_OPERATOR_LIKE:
-		case GDA_SQL_OPERATOR_GT:
-		case GDA_SQL_OPERATOR_LT:
-		case GDA_SQL_OPERATOR_GEQ:
-		case GDA_SQL_OPERATOR_LEQ:
-		case GDA_SQL_OPERATOR_DIFF:
-		case GDA_SQL_OPERATOR_REGEXP:
-		case GDA_SQL_OPERATOR_REGEXP_CI:
-		case GDA_SQL_OPERATOR_NOT_REGEXP:
-		case GDA_SQL_OPERATOR_NOT_REGEXP_CI:
-		case GDA_SQL_OPERATOR_SIMILAR:
-		case GDA_SQL_OPERATOR_REM:
-		case GDA_SQL_OPERATOR_DIV:
-		case GDA_SQL_OPERATOR_BITAND:
-		case GDA_SQL_OPERATOR_BITOR:
+		switch (operation->operator_type) {
+		case GDA_SQL_OPERATOR_TYPE_EQ:
+		case GDA_SQL_OPERATOR_TYPE_IS:
+		case GDA_SQL_OPERATOR_TYPE_LIKE:
+		case GDA_SQL_OPERATOR_TYPE_GT:
+		case GDA_SQL_OPERATOR_TYPE_LT:
+		case GDA_SQL_OPERATOR_TYPE_GEQ:
+		case GDA_SQL_OPERATOR_TYPE_LEQ:
+		case GDA_SQL_OPERATOR_TYPE_DIFF:
+		case GDA_SQL_OPERATOR_TYPE_REGEXP:
+		case GDA_SQL_OPERATOR_TYPE_REGEXP_CI:
+		case GDA_SQL_OPERATOR_TYPE_NOT_REGEXP:
+		case GDA_SQL_OPERATOR_TYPE_NOT_REGEXP_CI:
+		case GDA_SQL_OPERATOR_TYPE_SIMILAR:
+		case GDA_SQL_OPERATOR_TYPE_REM:
+		case GDA_SQL_OPERATOR_TYPE_DIV:
+		case GDA_SQL_OPERATOR_TYPE_BITAND:
+		case GDA_SQL_OPERATOR_TYPE_BITOR:
 			if (g_slist_length (operation->operands) != 2) {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 					     _("Wrong number of operands"));
 				return FALSE;
 			}
 			break;
-		case GDA_SQL_OPERATOR_BETWEEN:
+		case GDA_SQL_OPERATOR_TYPE_BETWEEN:
 			if (g_slist_length (operation->operands) != 3) {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 					     _("Wrong number of operands"));
 				return FALSE;
 			}
 			break;
-		case GDA_SQL_OPERATOR_BITNOT:
-		case GDA_SQL_OPERATOR_ISNULL:
-		case GDA_SQL_OPERATOR_ISNOTNULL:
-		case GDA_SQL_OPERATOR_NOT:
+		case GDA_SQL_OPERATOR_TYPE_BITNOT:
+		case GDA_SQL_OPERATOR_TYPE_ISNULL:
+		case GDA_SQL_OPERATOR_TYPE_ISNOTNULL:
+		case GDA_SQL_OPERATOR_TYPE_NOT:
 			if (g_slist_length (operation->operands) != 1) {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 					     _("Wrong number of operands"));
 				return FALSE;
 			}
 			break;
-		case GDA_SQL_OPERATOR_AND:
-		case GDA_SQL_OPERATOR_OR:
-		case GDA_SQL_OPERATOR_IN:
-		case GDA_SQL_OPERATOR_NOTIN:
-		case GDA_SQL_OPERATOR_CONCAT:
-		case GDA_SQL_OPERATOR_STAR:
+		case GDA_SQL_OPERATOR_TYPE_AND:
+		case GDA_SQL_OPERATOR_TYPE_OR:
+		case GDA_SQL_OPERATOR_TYPE_IN:
+		case GDA_SQL_OPERATOR_TYPE_NOTIN:
+		case GDA_SQL_OPERATOR_TYPE_CONCAT:
+		case GDA_SQL_OPERATOR_TYPE_STAR:
 			if (g_slist_length (operation->operands) < 2) {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 					     _("Wrong number of operands"));
 				return FALSE;
 			}
 			break;
-		case GDA_SQL_OPERATOR_MINUS:
-		case GDA_SQL_OPERATOR_PLUS:
+		case GDA_SQL_OPERATOR_TYPE_MINUS:
+		case GDA_SQL_OPERATOR_TYPE_PLUS:
 			if (g_slist_length (operation->operands) == 0) {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 					     _("Wrong number of operands"));
@@ -672,7 +994,7 @@ gda_sql_any_part_check_structure (GdaSqlAnyPart *node, GError **error)
 			break;
 		default:
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-				     _("Unknown operator %d"), operation->operator);
+				     _("Unknown operator %d"), operation->operator_type);
 			return FALSE;
 		}
 		break;
@@ -711,12 +1033,12 @@ gda_sql_any_part_check_structure (GdaSqlAnyPart *node, GError **error)
 	}
 	case GDA_SQL_ANY_SQL_SELECT_JOIN: {
 		GdaSqlSelectJoin *join = (GdaSqlSelectJoin*) node;
-		if (join->expr && join->using) {
+		if (join->expr && join->use) {
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 				     _("Join can't at the same time specify a join condition and a list of fields to join on"));
 			return FALSE;
 		}
-		if ((join->type == GDA_SQL_SELECT_JOIN_CROSS) && (join->expr || join->using)) {
+		if ((join->type == GDA_SQL_SELECT_JOIN_CROSS) && (join->expr || join->use)) {
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 				     _("Cross join can't have a join condition or a list of fields to join on"));
 			return FALSE;
@@ -917,7 +1239,7 @@ gda_sql_any_part_foreach (GdaSqlAnyPart *node, GdaSqlForeachFunc func, gpointer
 		GdaSqlSelectJoin *join = (GdaSqlSelectJoin *) node;
 		if (!gda_sql_any_part_foreach (GDA_SQL_ANY_PART (join->expr), func, data, error))
 			return FALSE;
-		for (l = join->using; l; l = l->next)
+		for (l = join->use; l; l = l->next)
 			if (!gda_sql_any_part_foreach (GDA_SQL_ANY_PART (l->data), func, data, error))
 				return FALSE;
 		break;
@@ -945,3 +1267,98 @@ gda_sql_any_part_foreach (GdaSqlAnyPart *node, GdaSqlForeachFunc func, gpointer
 	/* finally call @func for this node */
 	return func (node, data, error);
 }
+
+
+static gboolean foreach_normalize (GdaSqlAnyPart *node, GdaConnection *cnc, GError **error);
+
+/**
+ * gda_sql_statement_normalize
+ * @stmt: a pointer to a #GdaSqlStatement structure
+ * @cnc: a #GdaConnection object, or %NULL
+ *
+ * "Normalizes" (in place) some parts of @stmt, which means @stmt may be modified.
+ * At the moment any "*" field in a SELECT statement will be replaced by one
+ * #GdaSqlSelectField structure for each field in the referenced table.
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_sql_statement_normalize (GdaSqlStatement *stmt, GdaConnection *cnc, GError **error)
+{
+	gboolean retval;
+	g_return_val_if_fail (stmt, FALSE);
+
+	if (!stmt->validity_meta_struct && !gda_sql_statement_check_validity (stmt, cnc, error))
+		return FALSE;
+
+	retval = gda_sql_any_part_foreach (GDA_SQL_ANY_PART (stmt->contents), 
+					   (GdaSqlForeachFunc) foreach_normalize, cnc, error);
+#ifdef GDA_DEBUG
+	GError *lerror = NULL;
+	if (retval && !gda_sql_statement_check_validity (stmt, cnc, &lerror)) {
+		g_warning ("Internal error in %s(): statement is not valid anymore after: %s", __FUNCTION__,
+			   lerror && lerror->message ? lerror->message :  "No detail");
+		if (lerror)
+			g_error_free (lerror);
+	}
+#endif
+	return retval;
+}
+
+static gboolean
+foreach_normalize (GdaSqlAnyPart *node, GdaConnection *cnc, GError **error)
+{
+	if (!node) return TRUE;
+
+	if (node->type == GDA_SQL_ANY_SQL_SELECT_FIELD) {
+		GdaSqlSelectField *field = (GdaSqlSelectField*) node;
+		if (((field->field_name && gda_identifier_equal (field->field_name, "*")) ||
+		    (field->expr && field->expr->value && (G_VALUE_TYPE (field->expr->value) == G_TYPE_STRING) &&
+		     gda_identifier_equal (g_value_get_string (field->expr->value), "*"))) &&
+		    field->validity_meta_object) {
+			/* expand * to all the fields */
+			GdaMetaTable *mtable = GDA_META_TABLE (field->validity_meta_object);
+			GSList *list;
+			GdaSqlAnyPart *parent_node = ((GdaSqlAnyPart*) field)->parent;
+			gint nodepos = g_slist_index (((GdaSqlStatementSelect*) parent_node)->expr_list, node);
+			if (parent_node->type != GDA_SQL_ANY_STMT_SELECT) {
+				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+					     _("Select field is not in a SELECT statement"));
+				return FALSE;
+			}
+			for (list = mtable->columns; list; list = list->next) {
+				GdaSqlSelectField *nfield;
+				GdaMetaTableColumn *tcol = (GdaMetaTableColumn *) list->data;
+
+				nfield = gda_sql_select_field_new (parent_node);
+				nfield->field_name = g_strdup (tcol->column_name);
+				if (field->table_name)
+					nfield->table_name = g_strdup (field->table_name);
+				nfield->validity_meta_object = field->validity_meta_object;
+				nfield->validity_meta_table_column = tcol;
+				nfield->expr = gda_sql_expr_new ((GdaSqlAnyPart*) nfield);
+				nfield->expr->value = gda_value_new (G_TYPE_STRING);
+				if (field->table_name)
+					g_value_take_string (nfield->expr->value, g_strdup_printf ("%s.%s", 
+												   nfield->table_name,
+												   nfield->field_name));
+				else
+					g_value_set_string (nfield->expr->value, nfield->field_name);
+
+				/* insert nfield into expr_list */
+				GSList *expr_list = ((GdaSqlStatementSelect*) parent_node)->expr_list;
+				if (list == mtable->columns) {
+					GSList *lnode = g_slist_nth (expr_list, nodepos);
+					lnode->data = nfield;
+				}
+				else 
+					((GdaSqlStatementSelect*) parent_node)->expr_list = 
+						g_slist_insert (expr_list, nfield, ++nodepos);
+			}
+			/* get rid of @field */
+			gda_sql_select_field_free (field);
+		}
+	}
+
+	return TRUE;
+}
