@@ -73,14 +73,23 @@ typedef enum {
         /* I_STMT_DOMAINS_ALL, */
         /* I_STMT_DOMAINS_CONSTRAINTS, */
         /* I_STMT_DOMAINS_CONSTRAINTS_ALL, */
+        I_STMT_CHARACTER_SETS,
+        I_STMT_CHARACTER_SETS_ALL,
         I_STMT_VIEWS_COLUMNS,
         I_STMT_VIEWS_COLUMNS_ALL,
         I_STMT_TRIGGERS,
-        I_STMT_TRIGGERS_ALL/* , */
+        I_STMT_TRIGGERS_ALL,
         /* I_STMT_EL_TYPES_COL, */
         /* I_STMT_EL_TYPES_DOM, */
         /* I_STMT_EL_TYPES_UDT, */
-        /* I_STMT_EL_TYPES_ALL */
+        /* I_STMT_EL_TYPES_ALL, */
+        I_STMT_ROUTINES_ALL,
+        I_STMT_ROUTINES,
+        I_STMT_ROUTINES_ONE,
+        I_STMT_ROUTINES_PAR_ALL,
+        I_STMT_ROUTINES_PAR/* , */
+        /* I_STMT_ROUTINES_COL_ALL, */
+        /* I_STMT_ROUTINES_COL */
 } InternalStatementItem;
 
 
@@ -94,60 +103,66 @@ static gchar *internal_sql[] = {
         /* I_STMT_BTYPES */
 
         /* I_STMT_SCHEMAS */
-	"SELECT IFNULL(catalog_name, schema_name), schema_name, NULL, CASE schema_name WHEN 'information_schema' THEN 1 ELSE 0 END FROM information_schema.schemata WHERE schema_name = ##cat::string",
+	"SELECT IFNULL(catalog_name, schema_name) AS catalog_name, schema_name, NULL, CASE schema_name WHEN 'information_schema' OR 'mysql' THEN TRUE ELSE FALSE END AS schema_internal FROM INFORMATION_SCHEMA.schemata WHERE schema_name = ##cat::string",
 
         /* I_STMT_SCHEMAS_ALL */
-	"SELECT IFNULL(catalog_name, schema_name), schema_name, NULL, CASE schema_name WHEN 'information_schema' THEN 1 ELSE 0 END FROM information_schema.schemata",
+	"SELECT IFNULL(catalog_name, schema_name) AS catalog_name, schema_name, NULL, CASE schema_name WHEN 'information_schema' OR 'mysql' THEN TRUE ELSE FALSE END AS schema_internal FROM INFORMATION_SCHEMA.schemata",
 
         /* I_STMT_SCHEMA_NAMED */
-	"SELECT IFNULL(catalog_name, schema_name), schema_name, NULL, CASE schema_name WHEN 'information_schema' THEN 1 ELSE 0 END FROM information_schema.schemata WHERE schema_name = ##name::string",
+	"SELECT IFNULL(catalog_name, schema_name) AS catalog_name, schema_name, NULL, CASE schema_name WHEN 'information_schema' OR 'mysql' THEN TRUE ELSE FALSE END AS schema_internal FROM INFORMATION_SCHEMA.schemata WHERE schema_name = ##name::string",
 
         /* I_STMT_TABLES */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, table_type, CASE table_type WHEN 'BASE TABLE' THEN 1 ELSE 0 END, table_comment, table_name, CONCAT(table_schema, '.', table_name), NULL FROM information_schema.tables WHERE table_catalog = ##cat::string AND schema_name = ##schema::string",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, table_type, CASE table_type WHEN 'BASE TABLE' THEN TRUE ELSE FALSE END AS table_type, table_comment, table_name, CONCAT(table_schema, '.', table_name) AS table_full_name, NULL AS table_owner FROM INFORMATION_SCHEMA.tables WHERE table_catalog = ##cat::string AND schema_name = ##schema::string",
 
         /* I_STMT_TABLES_ALL */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, table_type, CASE table_type WHEN 'BASE TABLE' THEN 1 ELSE 0 END, table_comment, table_name, CONCAT(table_schema, '.', table_name), NULL FROM information_schema.tables",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, table_type, CASE table_type WHEN 'BASE TABLE' THEN TRUE ELSE FALSE END AS table_type, table_comment, table_name, CONCAT(table_schema, '.', table_name) AS table_full_name, NULL AS table_owner FROM INFORMATION_SCHEMA.tables",
 
         /* I_STMT_TABLE_NAMED */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, table_type, CASE table_type WHEN 'BASE TABLE' THEN 1 ELSE 0 END, table_comment, table_name, CONCAT(table_schema, '.', table_name), NULL FROM information_schema.tables WHERE table_catalog = ##cat::string AND schema_name = ##schema::string",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, table_type, CASE table_type WHEN 'BASE TABLE' THEN TRUE ELSE FALSE END AS table_type, table_comment, table_name, CONCAT(table_schema, '.', table_name) AS table_full_name, NULL AS table_owner FROM FROM INFORMATION_SCHEMA.tables WHERE table_catalog = ##cat::string AND schema_name = ##schema::string",
 
         /* I_STMT_VIEWS */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, view_definition, check_option, is_updatable FROM information_schema.views ",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, view_definition, check_option, is_updatable FROM INFORMATION_SCHEMA.views ",
 
         /* I_STMT_VIEWS_ALL */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, view_definition, check_option, is_updatable FROM information_schema.views ",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, view_definition, check_option, is_updatable FROM INFORMATION_SCHEMA.views ",
 
         /* I_STMT_VIEW_NAMED */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, view_definition, check_option, is_updatable FROM information_schema.views WHERE table_catalog = ##cat::string AND schema_name = ##schema::string",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, view_definition, check_option, is_updatable FROM INFORMATION_SCHEMA.views WHERE table_catalog = ##cat::string AND schema_name = ##schema::string",
 
         /* I_STMT_COLUMNS_OF_TABLE */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, column_name, ordinal_position, column_default, is_nullable, data_type, 'gchararray', character_maximum_length,character_octet_length, numeric_precision, numeric_scale, 0, character_set_name, character_set_name, character_set_name, collation_name, collation_name, collation_name, '', 1, column_comment FROM information_schema.columns WHERE table_catalog = ##cat::string AND table_schema = ##schema::string AND table_name = ##name::string",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, column_name, ordinal_position, column_default, is_nullable, data_type, 'gchararray', character_maximum_length,character_octet_length, numeric_precision, numeric_scale, 0, character_set_name, character_set_name, character_set_name, collation_name, collation_name, collation_name, extra, 1, column_comment FROM INFORMATION_SCHEMA.columns WHERE table_catalog = ##cat::string AND table_schema = ##schema::string AND table_name = ##name::string",
 
         /* I_STMT_COLUMNS_ALL */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, column_name, ordinal_position, column_default, is_nullable, data_type, 'gchararray', character_maximum_length,character_octet_length, numeric_precision, numeric_scale, 0, character_set_name, character_set_name, character_set_name, collation_name, collation_name, collation_name, '', 1, column_comment FROM information_schema.columns",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, column_name, ordinal_position, column_default, CASE is_nullable WHEN 'YES' THEN TRUE ELSE FALSE END AS is_nullable, data_type, 'gchararray', '', character_maximum_length, character_octet_length, numeric_precision, numeric_scale, 0, character_set_name, character_set_name, character_set_name, collation_name, collation_name, collation_name, extra, IF(FIND_IN_SET('insert', privileges) != 0 OR FIND_IN_SET('update', privileges) != 0, TRUE, FALSE) AS is_updatable, column_comment FROM INFORMATION_SCHEMA.columns",
 
         /* I_STMT_TABLES_CONSTRAINTS */
-	"SELECT IFNULL(constraint_catalog, constraint_schema), constraint_schema, constraint_name, IFNULL(table_catalog, table_schema), table_schema, table_name, constraint_type, NULL, 0, 0 FROM information_schema.table_constraints WHERE constraint_catalog = ##cat::string AND constraint_schema = ##schema::string AND table_name = ##name::string",
+	"SELECT IFNULL(constraint_catalog, constraint_schema) AS constraint_catalog, constraint_schema, constraint_name, IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, constraint_type, NULL, FALSE, FALSE FROM INFORMATION_SCHEMA.table_constraints WHERE constraint_catalog = ##cat::string AND constraint_schema = ##schema::string AND table_name = ##name::string",
 
         /* I_STMT_TABLES_CONSTRAINTS_ALL */
-	"SELECT IFNULL(constraint_catalog, constraint_schema), constraint_schema, constraint_name, IFNULL(constraint_catalog, constraint_schema), table_schema, table_name, constraint_type, NULL, 0, 0 FROM information_schema.table_constraints",
+	"SELECT IFNULL(constraint_catalog, constraint_schema) AS constraint_catalog, constraint_schema, constraint_name, IFNULL(constraint_catalog, constraint_schema) AS onstraint_catalog, table_schema, table_name, constraint_type, NULL, FALSE, FALSE FROM INFORMATION_SCHEMA.table_constraints",
 
         /* I_STMT_TABLES_CONSTRAINTS_NAMED */
-	"SELECT IFNULL(constraint_catalog, constraint_schema), constraint_schema, constraint_name, IFNULL(table_catalog, table_schema), table_schema, table_name, constraint_type, NULL, 0, 0 FROM information_schema.table_constraints WHERE constraint_catalog = ##cat::string AND constraint_schema = ##schema::string AND table_name = ##name::string AND constraint_name = ##name2::string",
+	"SELECT IFNULL(constraint_catalog, constraint_schema) AS constraint_catalog, constraint_schema, constraint_name, IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, constraint_type, NULL, FALSE, FALSE FROM INFORMATION_SCHEMA.table_constraints WHERE constraint_catalog = ##cat::string AND constraint_schema = ##schema::string AND table_name = ##name::string AND constraint_name = ##name2::string",
 
 #if MYSQL_VERSION_ID >= 50110
         /* I_STMT_REF_CONSTRAINTS */
-	"SELECT IFNULL(t.constraint_catalog, t.constraint_schema), t.constraint_schema, r.constraint_name, IFNULL(r.constraint_catalog, r.constraint_schema), r.constraint_schema, r.match_option, r.update_rule, delete_rule FROM information_schema.referential_constraint r INNER JOIN information_schema.table_constraints t ON r.constraint_schema=t.constraint_schema AND r.constraint_name=t.constraint_name AND r.table_name=t.table_name WHERE r.constraint_catalog = ##cat::string AND r.constraint_schema = ##schema::string AND r.table_name = ##name AND r.constraint_name = ##name2::string",
+	"SELECT IFNULL(t.constraint_catalog, t.constraint_schema) AS t.constraint_catalog, t.constraint_schema, r.constraint_name, IFNULL(r.constraint_catalog, r.constraint_schema) AS r.constraint_catalog, r.constraint_schema, r.match_option, r.update_rule, delete_rule FROM INFORMATION_SCHEMA.referential_constraint r INNER JOIN INFORMATION_SCHEMA.table_constraints t ON r.constraint_schema=t.constraint_schema AND r.constraint_name=t.constraint_name AND r.table_name=t.table_name WHERE r.constraint_catalog = ##cat::string AND r.constraint_schema = ##schema::string AND r.table_name = ##name AND r.constraint_name = ##name2::string",
 
         /* I_STMT_REF_CONSTRAINTS_ALL */
-	"SELECT IFNULL(t.constraint_catalog, t.constraint_schema), t.constraint_schema, r.constraint_name, IFNULL(r.constraint_catalog, r.constraint_schema), r.constraint_schema, r.match_option, r.update_rule, delete_rule FROM information_schema.referential_constraint r INNER JOIN information_schema.table_constraints t ON r.constraint_schema=t.constraint_schema AND r.constraint_name=t.constraint_name AND r.table_name=t.table_name",
+	"SELECT IFNULL(t.constraint_catalog, t.constraint_schema) AS t.constraint_catalog, t.constraint_schema, r.constraint_name, IFNULL(r.constraint_catalog, r.constraint_schema) AS r.constraint_catalog, r.constraint_schema, r.match_option, r.update_rule, delete_rule FROM INFORMATION_SCHEMA.referential_constraint r INNER JOIN INFORMATION_SCHEMA.table_constraints t ON r.constraint_schema=t.constraint_schema AND r.constraint_name=t.constraint_name AND r.table_name=t.table_name",
 #endif
 
         /* I_STMT_KEY_COLUMN_USAGE */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, constraint_name, column_name FROM information_schema.key_column_usage WHERE table_catalog = ##cat::string AND table_schema = ##schema::string AND table_name = ##name::string AND constraint_name = ##name2::string",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, constraint_name, column_name, ordinal_position FROM INFORMATION_SCHEMA.key_column_usage WHERE table_catalog = ##cat::string AND table_schema = ##schema::string AND table_name = ##name::string AND constraint_name = ##name2::string",
 
         /* I_STMT_KEY_COLUMN_USAGE_ALL */
-	"SELECT IFNULL(table_catalog, table_schema), table_schema, table_name, constraint_name, column_name, ordinal_position FROM information_schema.key_column_usage",
+	"SELECT IFNULL(table_catalog, table_schema) AS table_catalog, table_schema, table_name, constraint_name, column_name, ordinal_position FROM INFORMATION_SCHEMA.key_column_usage",
+
+        /* I_STMT_CHARACTER_SETS */
+	"SELECT DATABASE() AS character_set_catalog, DATABASE() AS character_set_schema, character_set_name, default_collate_name, default_collate_name, default_collate_name, description, character_set_name, character_set_name FROM INFORMATION_SCHEMA.character_sets WHERE character_set_catalog = ##cat::string AND character_set_schema = ##schema::string AND character_set_name = ##name::string",
+
+        /* I_STMT_CHARACTER_SETS_ALL */
+	"SELECT DATABASE() AS character_set_catalog, DATABASE() AS character_set_schema, character_set_name, default_collate_name, default_collate_name, default_collate_name, description, character_set_name, character_set_name FROM INFORMATION_SCHEMA.character_sets",
 
         /* I_STMT_UDT */
 
@@ -166,16 +181,16 @@ static gchar *internal_sql[] = {
         /* I_STMT_DOMAINS_CONSTRAINTS_ALL */
 
         /* I_STMT_VIEWS_COLUMNS */
-	"SELECT IFNULL(v.table_catalog, v.table_schema), v.table_schema, v.table_name, IFNULL(c.table_catalog, c.table_schema), c.table_schema, c.table_name, c.column_name FROM information_schema.columns c INNER JOIN information_schema.views v ON c.table_schema=v.table_schema AND c.table_name=v.table_name WHERE v.table_catalog = ##cat::string AND v.table_schema = ##schema::string AND v.table_name = ##name::string",
+	"SELECT IFNULL(v.table_catalog, v.table_schema) AS table_catalog, v.table_schema, v.table_name, IFNULL(c.table_catalog, c.table_schema) AS table_catalog, c.table_schema, c.table_name, c.column_name FROM INFORMATION_SCHEMA.columns c INNER JOIN INFORMATION_SCHEMA.views v ON c.table_schema=v.table_schema AND c.table_name=v.table_name WHERE v.table_catalog = ##cat::string AND v.table_schema = ##schema::string AND v.table_name = ##name::string",
 
         /* I_STMT_VIEWS_COLUMNS_ALL */
-	"SELECT IFNULL(v.table_catalog, v.table_schema), v.table_schema, v.table_name, IFNULL(c.table_catalog, c.table_schema), c.table_schema, c.table_name, c.column_name FROM information_schema.columns c INNER JOIN information_schema.views v ON c.table_schema=v.table_schema AND c.table_name=v.table_name",
+	"SELECT IFNULL(v.table_catalog, v.table_schema) AS table_catalog, v.table_schema, v.table_name, IFNULL(c.table_catalog, c.table_schema) AS table_catalog, c.table_schema, c.table_name, c.column_name FROM INFORMATION_SCHEMA.columns c INNER JOIN INFORMATION_SCHEMA.views v ON c.table_schema=v.table_schema AND c.table_name=v.table_name",
 
         /* I_STMT_TRIGGERS */
-	"SELECT IFNULL(trigger_catalog, trigger_schema), trigger_schema, trigger_name, event_manipulation, IFNULL(event_object_catalog, event_object_schema), event_object_schema, event_object_table, action_statement, action_orientation, action_timing, NULL, trigger_name, trigger_name FROM information_schema.triggers WHERE trigger_catalog = ##cat::string AND trigger_schema =  ##schema::string AND trigger_name = ##name::string",
+	"SELECT IFNULL(trigger_catalog, trigger_schema) AS trigger_catalog, trigger_schema, trigger_name, event_manipulation, IFNULL(event_object_catalog, event_object_schema) AS event_object_catalog, event_object_schema, event_object_table, action_statement, action_orientation, action_timing, NULL, trigger_name, trigger_name FROM INFORMATION_SCHEMA.triggers WHERE trigger_catalog = ##cat::string AND trigger_schema =  ##schema::string AND trigger_name = ##name::string",
 
         /* I_STMT_TRIGGERS_ALL */
-	"SELECT IFNULL(trigger_catalog, trigger_schema), trigger_schema, trigger_name, event_manipulation, IFNULL(event_object_catalog, event_object_schema), event_object_schema, event_object_table, action_statement, action_orientation, action_timing, NULL, trigger_name, trigger_name FROM information_schema.triggers",
+	"SELECT IFNULL(trigger_catalog, trigger_schema) AS trigger_catalog, trigger_schema, trigger_name, event_manipulation, IFNULL(event_object_catalog, event_object_schema) AS event_object_catalog, event_object_schema, event_object_table, action_statement, action_orientation, action_timing, NULL, trigger_name, trigger_name FROM INFORMATION_SCHEMA.triggers",
 
         /* I_STMT_EL_TYPES_COL */
 
@@ -185,6 +200,25 @@ static gchar *internal_sql[] = {
 
         /* I_STMT_EL_TYPES_ALL */
 
+        /* I_STMT_ROUTINES_ALL */
+	"SELECT IFNULL(routine_catalog, routine_schema) AS specific_catalog, routine_schema AS specific_schema, routine_name AS specific_name, IFNULL(routine_catalog, routine_schema) AS routine_catalog, routine_schema, routine_name, routine_type, dtd_identifier AS return_type, FALSE AS returns_set, 0 AS nb_args, routine_body, routine_definition, external_name, external_language, parameter_style, CASE is_deterministic WHEN 'YES' THEN TRUE ELSE FALSE END AS is_deterministic, sql_data_access, FALSE AS is_null_call, routine_comment, routine_name, routine_name, definer FROM INFORMATION_SCHEMA.routines",
+
+        /* I_STMT_ROUTINES */
+	"SELECT IFNULL(routine_catalog, routine_schema) AS specific_catalog, routine_schema AS specific_schema, routine_name AS specific_name, IFNULL(routine_catalog, routine_schema) AS routine_catalog, routine_schema, routine_name, routine_type, dtd_identifier AS return_type, FALSE AS returns_set, 0 AS nb_args, routine_body, routine_definition, external_name, external_language, parameter_style, CASE is_deterministic WHEN 'YES' THEN TRUE ELSE FALSE END AS is_deterministic, sql_data_access, FALSE AS is_null_call, routine_comment, routine_name, routine_name, definer FROM INFORMATION_SCHEMA.routines WHERE routine_catalog = ##cat::string AND routine_schema =  ##schema::string",
+
+        /* I_STMT_ROUTINES_ONE */
+	"SELECT IFNULL(routine_catalog, routine_schema) AS specific_catalog, routine_schema AS specific_schema, routine_name AS specific_name, IFNULL(routine_catalog, routine_schema) AS routine_catalog, routine_schema, routine_name, routine_type, dtd_identifier AS return_type, FALSE AS returns_set, 0 AS nb_args, routine_body, routine_definition, external_name, external_language, parameter_style, CASE is_deterministic WHEN 'YES' THEN TRUE ELSE FALSE END AS is_deterministic, sql_data_access, FALSE AS is_null_call, routine_comment, routine_name, routine_name, definer FROM INFORMATION_SCHEMA.routines WHERE routine_catalog = ##cat::string AND routine_schema =  ##schema::string AND routine_name = ##name::string",
+
+        /* I_STMT_ROUTINES_PAR_ALL */
+	"SELECT IFNULL(routine_catalog, routine_schema) AS specific_catalog, routine_schema AS specific_schema, routine_name AS specific_name, IFNULL(routine_catalog, routine_schema) AS routine_catalog, routine_schema, routine_name, routine_type, dtd_identifier AS return_type, FALSE AS returns_set, 0 AS nb_args, routine_body, routine_definition, external_name, external_language, parameter_style, CASE is_deterministic WHEN 'YES' THEN TRUE ELSE FALSE END AS is_deterministic, sql_data_access, FALSE AS is_null_call, routine_comment, routine_name, routine_name, definer FROM INFORMATION_SCHEMA.routines",
+
+        /* I_STMT_ROUTINES_PAR */
+	"SELECT IFNULL(routine_catalog, routine_schema) AS specific_catalog, routine_schema AS specific_schema, routine_name AS specific_name, IFNULL(routine_catalog, routine_schema) AS routine_catalog, routine_schema, routine_name, routine_type, dtd_identifier AS return_type, FALSE AS returns_set, 0 AS nb_args, routine_body, routine_definition, external_name, external_language, parameter_style, CASE is_deterministic WHEN 'YES' THEN TRUE ELSE FALSE END AS is_deterministic, sql_data_access, FALSE AS is_null_call, routine_comment, routine_name, routine_name, definer FROM INFORMATION_SCHEMA.routines WHERE routine_catalog = ##cat::string AND routine_schema =  ##schema::string AND routine_name = ##name::string",
+
+        /* I_STMT_ROUTINES_COL_ALL */
+
+        /* I_STMT_ROUTINES_COL */
+
 };
 
 /*
@@ -217,9 +251,10 @@ _gda_mysql_provider_meta_init (GdaServerProvider  *provider)
         }
 
 	/* initialize static values here */
-	i_set = gda_set_new_inline (3, "cat", G_TYPE_STRING, "", 
+	i_set = gda_set_new_inline (4, "cat", G_TYPE_STRING, "", 
 				    "name", G_TYPE_STRING, "",
-				    "schema", G_TYPE_STRING, "");
+				    "schema", G_TYPE_STRING, "",
+                                    "name2", G_TYPE_STRING, "");
 
 	g_static_mutex_unlock (&init_mutex);
 }
@@ -302,7 +337,7 @@ _gda_mysql_meta__btypes (GdaServerProvider  *prov,
 		for (i = 0; i < sizeof(data_types) / sizeof(BuiltinDataType); ++i) {
 			BuiltinDataType *data_type = &(data_types[i]);
 			GList *values = NULL;
-			GValue *tmp_value = { 0, };
+			GValue *tmp_value = NULL;
 
 			g_value_set_string (tmp_value = gda_value_new (G_TYPE_STRING), data_type->tname);
 			values = g_list_append (values, tmp_value); 
@@ -399,7 +434,8 @@ _gda_mysql_meta__enums (GdaServerProvider  *prov,
 			GdaMetaContext     *context,
 			GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;	
 }
 
@@ -413,7 +449,8 @@ _gda_mysql_meta_enums (GdaServerProvider  *prov,
 		       const GValue       *udt_schema,
 		       const GValue       *udt_name)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;
 }
 
@@ -425,7 +462,8 @@ _gda_mysql_meta__domains (GdaServerProvider  *prov,
 			  GdaMetaContext     *context,
 			  GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;
 }
 
@@ -438,7 +476,8 @@ _gda_mysql_meta_domains (GdaServerProvider  *prov,
 			 const GValue       *domain_catalog,
 			 const GValue       *domain_schema)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;
 }
 
@@ -449,7 +488,8 @@ _gda_mysql_meta__constraints_dom (GdaServerProvider  *prov,
 				  GdaMetaContext     *context,
 				  GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;
 }
 
@@ -463,7 +503,8 @@ _gda_mysql_meta_constraints_dom (GdaServerProvider  *prov,
 				 const GValue       *domain_schema, 
 				 const GValue       *domain_name)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;
 }
 
@@ -474,10 +515,24 @@ _gda_mysql_meta__el_types (GdaServerProvider  *prov,
 			   GdaMetaContext     *context,
 			   GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;
 }
 
+gboolean
+_gda_mysql_meta_el_types (GdaServerProvider  *prov,
+			  GdaConnection      *cnc,
+			  GdaMetaStore       *store,
+			  GdaMetaContext     *context,
+			  GError            **error,
+			  const GValue       *specific_name)
+{
+        //TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
+        return TRUE;
+}
+
 gboolean
 _gda_mysql_meta__collations (GdaServerProvider  *prov,
 			     GdaConnection      *cnc, 
@@ -485,7 +540,8 @@ _gda_mysql_meta__collations (GdaServerProvider  *prov,
 			     GdaMetaContext     *context,
 			     GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
+	/* Feature not supported by MySQL. */
 	return TRUE;
 }
 
@@ -510,8 +566,19 @@ _gda_mysql_meta__character_sets (GdaServerProvider  *prov,
 				 GdaMetaContext     *context,
 				 GError            **error)
 {
-	TO_IMPLEMENT;
-	return TRUE;
+	//TO_IMPLEMENT;
+
+	GdaDataModel *model;
+	gboolean retval;
+	model = gda_connection_statement_execute_select	(cnc, internal_stmt[I_STMT_CHARACTER_SETS_ALL], NULL, error);
+	if (model == NULL)
+		retval = FALSE;
+	else {
+		retval = gda_meta_store_modify_with_context (store, context, model, error);
+		g_object_unref (G_OBJECT(model));
+	}
+
+	return retval;
 }
 
 gboolean
@@ -524,8 +591,27 @@ _gda_mysql_meta_character_sets (GdaServerProvider  *prov,
 				const GValue       *chset_schema, 
 				const GValue       *chset_name_n)
 {
-	TO_IMPLEMENT;
-	return TRUE;
+	//TO_IMPLEMENT;
+
+	GdaDataModel *model;
+	gboolean retval;
+
+	if (! gda_holder_set_value (gda_set_get_holder (i_set, "cat"), chset_catalog, error))
+		return FALSE;
+	if (! gda_holder_set_value (gda_set_get_holder (i_set, "schema"), chset_schema, error))
+		return FALSE;
+	if (! gda_holder_set_value (gda_set_get_holder (i_set, "name"), chset_name_n, error))
+		return FALSE;
+	model = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_CHARACTER_SETS], i_set, error);
+	if (model == NULL)
+		retval = FALSE;
+	else {
+		retval = gda_meta_store_modify_with_context (store, context, model, error);
+		g_object_unref (G_OBJECT(model));
+
+	}
+
+	return retval;
 }
 
 gboolean
@@ -537,9 +623,14 @@ _gda_mysql_meta__schemata (GdaServerProvider  *prov,
 {
 	// TO_IMPLEMENT;
 
+	GType col_types[] = {
+		0, 0, 0,
+		G_TYPE_BOOLEAN, G_TYPE_NONE
+	};
 	GdaDataModel *model;
 	gboolean retval;
-	model = gda_connection_statement_execute_select	(cnc, internal_stmt[I_STMT_SCHEMAS_ALL], i_set, error);
+	model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_SCHEMAS_ALL], NULL,
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types, error);
 	if (model == NULL)
 		retval = FALSE;
 	else {
@@ -561,13 +652,18 @@ _gda_mysql_meta_schemata (GdaServerProvider  *prov,
 {
 	// TO_IMPLEMENT;
 	
+	GType col_types[] = {
+		0, 0, 0,
+		G_TYPE_BOOLEAN, G_TYPE_NONE
+	};
 	GdaDataModel *model;
 	gboolean retval;
 
 	if (! gda_holder_set_value (gda_set_get_holder (i_set, "cat"), catalog_name, error))
 		return FALSE;
 	if (!schema_name_n) {
-		model = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_SCHEMAS], i_set, error);
+		model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_SCHEMAS], i_set,
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types, error);
 		if (model == NULL)
 			retval = FALSE;
 		else {
@@ -577,7 +673,8 @@ _gda_mysql_meta_schemata (GdaServerProvider  *prov,
 	} else {
 		if (! gda_holder_set_value (gda_set_get_holder (i_set, "name"), schema_name_n, error))
 			return FALSE;
-		model = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_SCHEMA_NAMED], i_set, error);
+		model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_SCHEMA_NAMED], i_set,
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types, error);
 		if (model == NULL)
 			retval = FALSE;
 		else {
@@ -601,6 +698,10 @@ _gda_mysql_meta__tables_views (GdaServerProvider  *prov,
 {
 	// TO_IMPLEMENT;
 	
+	GType col_types_tables[] = {
+		0, 0, 0, 0,
+		G_TYPE_BOOLEAN, G_TYPE_NONE
+	};
 	GdaDataModel *model_tables, *model_views;
 	gboolean retval;
 	/* Check correct mysql server version. */
@@ -617,7 +718,8 @@ _gda_mysql_meta__tables_views (GdaServerProvider  *prov,
 	/* Copy contents, just because we need to modify @context->table_name */
 	GdaMetaContext copy = *context;
 
-	model_tables = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_TABLES_ALL], i_set, error);
+	model_tables = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_TABLES_ALL], NULL,
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types_tables, error);
 	if (model_tables == NULL)
 		retval = FALSE;
 	else {
@@ -626,7 +728,7 @@ _gda_mysql_meta__tables_views (GdaServerProvider  *prov,
 		g_object_unref (G_OBJECT(model_tables));
 	}
 
-	model_views = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_VIEWS_ALL], i_set, error);
+	model_views = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_VIEWS_ALL], NULL, error);
 	if (model_views == NULL)
 		retval = FALSE;
 	else {
@@ -715,7 +817,7 @@ _gda_mysql_meta_tables_views (GdaServerProvider  *prov,
 }
 
 /**
- * map_mysql_to_gda:
+ * map_mysql_type_to_gda:
  * @value: a #GValue string.
  *
  * It maps a mysql type to a gda type.  This means that when a
@@ -724,11 +826,12 @@ _gda_mysql_meta_tables_views (GdaServerProvider  *prov,
  * Returns a newly created GValue string.
  */
 static inline GValue *
-map_mysql_to_gda (const GValue  *value)
+map_mysql_type_to_gda (const GValue  *value)
 {
 	const gchar *string = g_value_get_string (value);
 	GValue *newvalue;
 	const gchar *newstring;
+
 	if (strcmp (string, "bool") == 0)
 		newstring = "gboolean";
 	else
@@ -835,8 +938,7 @@ map_mysql_to_gda (const GValue  *value)
 
 	g_value_set_string (newvalue = gda_value_new (G_TYPE_STRING),
 			    newstring);
-	
-	
+
 	return newvalue;
 }
 
@@ -849,10 +951,13 @@ _gda_mysql_meta__columns (GdaServerProvider  *prov,
 {
 	// TO_IMPLEMENT;
 
-	/* GType col_types[] = { */
-	/* 	G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, */
-	/* 	G_TYPE_INT, G_TYPE_NONE */
-	/* }; */
+	GType col_types[] = {
+		0, 0, 0, 0, G_TYPE_INT, G_TYPE_STRING,
+		G_TYPE_BOOLEAN,
+		0, 0, 0, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, 0, 0, 0, 0, 0, 0, 0,
+		G_TYPE_BOOLEAN,
+		G_TYPE_NONE
+	};
 	GdaDataModel *model, *proxy;
 	gboolean retval;
 	/* Check correct mysql server version. */
@@ -867,8 +972,8 @@ _gda_mysql_meta__columns (GdaServerProvider  *prov,
 	}
 
 	/* Use a prepared statement for the "base" model. */
-	model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_COLUMNS_ALL], i_set,
-							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, /* col_types */ NULL, error);
+	model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_COLUMNS_ALL], NULL,
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types, error);
 	if (model == NULL)
 		retval = FALSE;
 	else {
@@ -883,20 +988,19 @@ _gda_mysql_meta__columns (GdaServerProvider  *prov,
 				break;
 			}
 
-			GValue *newvalue = map_mysql_to_gda (value);
+			GValue *newvalue = map_mysql_type_to_gda (value);
 
-			retval = gda_data_model_set_value_at (GDA_DATA_MODEL(proxy), 8, i, newvalue, error);
+			retval = gda_data_model_set_value_at (GDA_DATA_MODEL(proxy), 9, i, newvalue, error);
 			gda_value_free (newvalue);
 			if (! retval)
 				break;
-
 		}
 
 		if (retval)
-			retval = gda_meta_store_modify_with_context (store, context, proxy, error);
+			retval = gda_meta_store_modify_with_context (store, context, proxy/* model */, error);
+
 		g_object_unref (G_OBJECT(proxy));
 		g_object_unref (G_OBJECT(model));
-
 	}
 
 	return retval;
@@ -914,10 +1018,13 @@ _gda_mysql_meta_columns (GdaServerProvider  *prov,
 {
 	// TO_IMPLEMENT;
 
-	/* GType col_types[] = { */
-	/* 	G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, */
-	/* 	G_TYPE_INT, G_TYPE_NONE */
-	/* }; */
+	GType col_types[] = {
+		0, 0, 0, 0, G_TYPE_INT, G_TYPE_STRING,
+		G_TYPE_BOOLEAN,
+		0, 0, 0, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, 0, 0, 0, 0, 0, 0, 0,
+		G_TYPE_BOOLEAN,
+		G_TYPE_NONE
+	};
 	GdaDataModel *model, *proxy;
 	gboolean retval;
 	/* Check correct mysql server version. */
@@ -939,7 +1046,7 @@ _gda_mysql_meta_columns (GdaServerProvider  *prov,
 	if (! gda_holder_set_value (gda_set_get_holder (i_set, "name"), table_name, error))
 		return FALSE;
 	model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_COLUMNS_OF_TABLE], i_set,
-							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, /* col_types */ NULL, error);
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types, error);
 	if (model == NULL)
 		retval = FALSE;
 	else {
@@ -954,13 +1061,12 @@ _gda_mysql_meta_columns (GdaServerProvider  *prov,
 				break;
 			}
 
-			GValue *newvalue = map_mysql_to_gda (value);
+			GValue *newvalue = map_mysql_type_to_gda (value);
 
-			retval = gda_data_model_set_value_at (GDA_DATA_MODEL(proxy), 8, i, newvalue, error);
+			retval = gda_data_model_set_value_at (GDA_DATA_MODEL(proxy), 9, i, newvalue, error);
 			gda_value_free (newvalue);
 			if (! retval)
 				break;
-
 		}
 
 		if (retval)
@@ -970,7 +1076,6 @@ _gda_mysql_meta_columns (GdaServerProvider  *prov,
 							"schema", table_schema, "name", table_name, NULL);
 		g_object_unref (G_OBJECT(proxy));
 		g_object_unref (G_OBJECT(model));
-
 	}
 
 	return retval;
@@ -998,7 +1103,7 @@ _gda_mysql_meta__view_cols (GdaServerProvider  *prov,
 		return FALSE;
 	}
 
-	model = gda_connection_statement_execute_select	(cnc, internal_stmt[I_STMT_VIEWS_COLUMNS_ALL], i_set, error);
+	model = gda_connection_statement_execute_select	(cnc, internal_stmt[I_STMT_VIEWS_COLUMNS_ALL], NULL, error);
 	if (model == NULL)
 		retval = FALSE;
 	else {
@@ -1050,9 +1155,14 @@ _gda_mysql_meta__constraints_tab (GdaServerProvider  *prov,
 {
 	// TO_IMPLEMENT;
 
+	GType col_types[] = {
+		0, 0, 0, 0, 0, 0, 0, 0,
+		G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_NONE
+	};
 	GdaDataModel *model;
 	gboolean retval;
-	model = gda_connection_statement_execute_select	(cnc, internal_stmt[I_STMT_TABLES_CONSTRAINTS_ALL], NULL, error);
+	model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_TABLES_CONSTRAINTS_ALL], NULL,
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types, error);
 	if (model == NULL)
 		retval = FALSE;
 	else {
@@ -1193,9 +1303,14 @@ _gda_mysql_meta__key_columns (GdaServerProvider  *prov,
 {
 	// TO_IMPLEMENT;
 
+	GType col_types[] = {
+		0, 0, 0, 0, 0,
+		G_TYPE_INT, G_TYPE_NONE
+	};
 	GdaDataModel *model;
 	gboolean retval;
-	model = gda_connection_statement_execute_select	(cnc, internal_stmt[I_STMT_KEY_COLUMN_USAGE_ALL], NULL, error);
+	model = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_STMT_KEY_COLUMN_USAGE_ALL], NULL,
+							      GDA_STATEMENT_MODEL_RANDOM_ACCESS, col_types, error);
 	if (model == NULL)
 		retval = FALSE;
 	else {
@@ -1263,7 +1378,7 @@ _gda_mysql_meta__check_columns (GdaServerProvider  *prov,
 				GdaMetaContext     *context,
 				GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
 	return TRUE;
 }
 
@@ -1278,7 +1393,7 @@ _gda_mysql_meta_check_columns (GdaServerProvider  *prov,
 			       const GValue       *table_name,
 			       const GValue       *constraint_name)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
 	return TRUE;
 }
 
@@ -1365,8 +1480,19 @@ _gda_mysql_meta__routines (GdaServerProvider  *prov,
 			   GdaMetaContext     *context,
 			   GError            **error)
 {
-	TO_IMPLEMENT;
-	return TRUE;
+	//TO_IMPLEMENT;
+
+	GdaDataModel *model;
+	gboolean retval;
+	model = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_ROUTINES_ALL], NULL, error);
+	if (model == NULL)
+		retval = FALSE;
+	else {
+		retval = gda_meta_store_modify_with_context (store, context, model, error);
+		g_object_unref (G_OBJECT(model));
+	}
+
+	return retval;
 }
 
 gboolean
@@ -1379,8 +1505,41 @@ _gda_mysql_meta_routines (GdaServerProvider  *prov,
 			  const GValue       *routine_schema, 
 			  const GValue       *routine_name_n)
 {
-	TO_IMPLEMENT;
-	return TRUE;
+	//TO_IMPLEMENT;
+	//
+
+	GdaDataModel *model;
+	gboolean retval;
+	/* Check correct mysql server version. */
+	MysqlConnectionData *cdata;
+	cdata = (MysqlConnectionData *) gda_connection_internal_get_provider_data (cnc);
+	if (!cdata)
+		return FALSE;
+	if (cdata->version_long < 50000) {
+		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_SERVER_VERSION_ERROR,
+			     _("Mysql version 5.0 at least is required"));
+		return FALSE;
+	}
+
+	if (! gda_holder_set_value (gda_set_get_holder (i_set, "cat"), routine_catalog, error))
+		return FALSE;
+	if (! gda_holder_set_value (gda_set_get_holder (i_set, "schema"), routine_schema, error))
+		return FALSE;
+	if (routine_name_n != NULL) {
+		if (! gda_holder_set_value (gda_set_get_holder (i_set, "name"), routine_name_n, error))
+			return FALSE;
+		model = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_ROUTINES_ONE], i_set, error);
+	} else
+		model = gda_connection_statement_execute_select (cnc, internal_stmt[I_STMT_ROUTINES], i_set, error);
+	if (model == NULL)
+		retval = FALSE;
+	else {
+		retval = gda_meta_store_modify_with_context (store, context, model, error);
+		g_object_unref (G_OBJECT(model));
+
+	}
+
+	return retval;
 }
 
 gboolean
@@ -1390,7 +1549,7 @@ _gda_mysql_meta__routine_col (GdaServerProvider  *prov,
 			      GdaMetaContext     *context,
 			      GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
 	return TRUE;
 }
 
@@ -1404,7 +1563,7 @@ _gda_mysql_meta_routine_col (GdaServerProvider  *prov,
 			     const GValue       *rout_schema, 
 			     const GValue       *rout_name)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
 	return TRUE;
 }
 
@@ -1415,7 +1574,7 @@ _gda_mysql_meta__routine_par (GdaServerProvider  *prov,
 			      GdaMetaContext     *context,
 			      GError            **error)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
 	return TRUE;
 }
 
@@ -1429,6 +1588,6 @@ _gda_mysql_meta_routine_par (GdaServerProvider  *prov,
 			     const GValue       *rout_schema, 
 			     const GValue       *rout_name)
 {
-	TO_IMPLEMENT;
+	//TO_IMPLEMENT;
 	return TRUE;
 }
