@@ -420,7 +420,6 @@ real_open_connection (const gchar  *host,
 	if (compress)
 		flags |= CLIENT_COMPRESS;
 	
-	
 	MYSQL *mysql = g_new0 (MYSQL, 1);
 	mysql_init (mysql);
 
@@ -516,8 +515,12 @@ gda_mysql_provider_open_connection (GdaServerProvider               *provider,
 	host = gda_quark_list_find (params, "HOST");
 
 	const gchar *user, *password;
-	user = gda_quark_list_find (params, "USER");
-	password = gda_quark_list_find (params, "PASSWORD");
+	user = gda_quark_list_find (auth, "USERNAME");
+	if (!user)
+		user = gda_quark_list_find (params, "USERNAME");
+	password = gda_quark_list_find (auth, "PASSWORD");
+	if (!password)
+		password = gda_quark_list_find (params, "PASSWORD");
 
 	const gchar *port, *unix_socket, *use_ssl, *compress;
 	port = gda_quark_list_find (params, "PORT");
@@ -525,7 +528,6 @@ gda_mysql_provider_open_connection (GdaServerProvider               *provider,
 	use_ssl = gda_quark_list_find (params, "USE_SSL");
 	compress = gda_quark_list_find (params, "COMPRESS");
 	
-	
 	/* open the real connection to the database */
 	/* TO_ADD: C API specific function calls;
 	 * if it fails, add a connection event and return FALSE */
@@ -554,7 +556,6 @@ gda_mysql_provider_open_connection (GdaServerProvider               *provider,
 		_gda_mysql_make_error (cnc, NULL, mysql_stmt, NULL);
 		return FALSE;
 	}
-	
 
 	/* Create a new instance of the provider specific data associated to a connection (MysqlConnectionData),
 	 * and set its contents */
@@ -570,7 +571,6 @@ gda_mysql_provider_open_connection (GdaServerProvider               *provider,
 	cdata->version_long = mysql_get_server_version (mysql);
 	cdata->version = get_mysql_version (mysql);
 	
-
 	/* Optionnally set some attributes for the newly opened connection (encoding to UTF-8 for example )*/
 	// TO_IMPLEMENT;
 
@@ -1068,7 +1068,7 @@ gda_mysql_provider_get_default_dbms_type (GdaServerProvider  *provider,
 	    (type == G_TYPE_UCHAR) ||
 	    (type == G_TYPE_ULONG) ||
 	    (type == G_TYPE_UINT) ||
-	    (type == G_TYPE_UINT64)) 
+	    (type == G_TYPE_UINT64))
 		return "integer";
 
 	if ((type == GDA_TYPE_BINARY) ||
@@ -1085,7 +1085,8 @@ gda_mysql_provider_get_default_dbms_type (GdaServerProvider  *provider,
 	    (type == G_TYPE_STRING) ||
 	    (type == GDA_TYPE_TIME) ||
 	    (type == GDA_TYPE_TIMESTAMP) ||
-	    (type == G_TYPE_INVALID))
+	    (type == G_TYPE_INVALID) ||
+	    (type == G_TYPE_GTYPE))
 		return "string";
 
 	if ((type == G_TYPE_DOUBLE) ||
@@ -1173,7 +1174,6 @@ gda_mysql_provider_statement_prepare (GdaServerProvider  *provider,
 
 	/* prepare @stmt using the C API, creates @ps */
 	// TO_IMPLEMENT;
-	
 
 	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
@@ -1188,7 +1188,6 @@ gda_mysql_provider_statement_prepare (GdaServerProvider  *provider,
 		(provider, cnc, stmt, set,
 		 GDA_STATEMENT_SQL_PARAMS_AS_UQMARK, &used_set, error);
 
-	
 	if (!sql)
 		goto cleanup;
 
@@ -1224,12 +1223,12 @@ gda_mysql_provider_statement_prepare (GdaServerProvider  *provider,
 	if (!ps)
 		return FALSE;
 	else {
-		
 		gda_pstmt_set_gda_statement (_GDA_PSTMT(ps), stmt);
 		_GDA_PSTMT(ps)->param_ids = param_ids;
 		_GDA_PSTMT(ps)->sql = sql;
 		
 		gda_connection_add_prepared_statement (cnc, stmt, (GdaPStmt*) ps);
+		g_object_unref (ps);
 		return TRUE;
 	}
 	
@@ -1255,7 +1254,7 @@ prepare_stmt_simple (MysqlConnectionData  *cdata,
 	
 	if (mysql_stmt_prepare (cdata->mysql_stmt, sql, strlen (sql))) {
 		GdaConnectionEvent *event = _gda_mysql_make_error
-			(cdata->cnc, NULL, cdata->mysql_stmt, NULL);
+			(cdata->cnc, NULL, cdata->mysql_stmt, error);
 		ps = NULL;
 	} else {
 		ps = gda_mysql_pstmt_new (cdata->cnc, cdata->mysql, cdata->mysql_stmt);
@@ -1295,6 +1294,9 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 {
 	GdaMysqlPStmt *ps;
 	MysqlConnectionData *cdata;
+	gboolean allow_noparam;
+        gboolean empty_rs = FALSE; /* TRUE when @allow_noparam is TRUE and there is a problem with @params
+                                      => resulting data model will be empty (0 row) */
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
@@ -1307,13 +1309,20 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
                 return FALSE;
 	}
 
-	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
-	if (!cdata) 
-		return FALSE;
+	if (! (model_usage & GDA_STATEMENT_MODEL_RANDOM_ACCESS) &&
+	    ! (model_usage & GDA_STATEMENT_MODEL_CURSOR_FORWARD))
+		model_usage |= GDA_STATEMENT_MODEL_RANDOM_ACCESS;
+
+	allow_noparam = (model_usage & GDA_STATEMENT_MODEL_ALLOW_NOPARAM) &&
+		(gda_statement_get_statement_type (stmt) == GDA_SQL_STATEMENT_SELECT);
 
 	if (last_inserted_row)
 		*last_inserted_row = NULL;
 
+	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	if (!cdata) 
+		return FALSE;
+
 	/* get/create new prepared statement */
 	ps = (GdaMysqlPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
 	if (!ps) {
@@ -1333,7 +1342,6 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			g_free (sql);
 			if (!ps)
 				return NULL;
-			
 		}
 		else
 			ps = (GdaMysqlPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
@@ -1352,10 +1360,10 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 	char **param_values = g_new0 (char *, nb_params + 1);
         int *param_lengths = g_new0 (int, nb_params + 1);
         int *param_formats = g_new0 (int, nb_params + 1);
-	/* g_print ("NB=%d, SQL=%s\n", nb_params, _GDA_PSTMT(ps)->sql); */
+	//g_print ("NB=%d, SQL=%s\n", nb_params, _GDA_PSTMT(ps)->sql);
 
 	MYSQL_BIND *mysql_bind_param = g_new0 (MYSQL_BIND, nb_params);
-	
+
 	for (i = 0, list = _GDA_PSTMT (ps)->param_ids; list; list = list->next, i++) {
 		const gchar *pname = (gchar *) list->data;
 
@@ -1378,24 +1386,43 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			}
 		}
 		if (!h) {
-			gchar *str;
-			str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
-			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
-			gda_connection_event_set_description (event, str);
-			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
-				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
-			g_free (str);
-			break;
+
+			if (!allow_noparam) {
+				gchar *str;
+				str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
+				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (event, str);
+				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
+				g_free (str);
+				break;
+			} else {
+                                /* bind param to NULL */
+                                //TO_IMPLEMENT;
+				param_values[i] = NULL;
+                                empty_rs = TRUE;
+                                continue;
+			}
 		}
+
 		if (!gda_holder_is_valid (h)) {
-			gchar *str;
-			str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
-			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
-			gda_connection_event_set_description (event, str);
-			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
-				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
-			g_free (str);
-			break;
+
+			if (!allow_noparam) {
+				gchar *str;
+				str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
+				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (event, str);
+				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
+				g_free (str);
+				break;
+			} else {
+                                /* bind param to NULL */
+				//TO_IMPLEMENT;
+				param_values[i] = NULL;
+                                empty_rs = TRUE;
+                                continue;
+                        }
 		}
 
 		/* actual binding using the C API, for parameter at position @i */
@@ -1407,31 +1434,27 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 		} else if ((G_VALUE_TYPE(value) == G_TYPE_DATE) ||
 			   (G_VALUE_TYPE(value) == GDA_TYPE_TIME) ||
 			   (G_VALUE_TYPE(value) == GDA_TYPE_TIMESTAMP)) {
-			
-			GdaHandlerTime *handler_time = (GdaHandlerTime *) gda_server_provider_get_data_handler_gtype
+			GdaHandlerTime *handler_time = (GdaHandlerTime *) gda_server_provider_get_data_handler_g_type
 				(provider, cnc, G_VALUE_TYPE(value));
 			g_assert (handler_time);
 			param_values[i] = gda_handler_time_get_no_locale_str_from_value (handler_time,
 											 value);
-			
-			/* g_print ("--- TIME=%s\n", param_values[i]); */
+			//g_print ("--- TIME=%s\n", param_values[i]);
 		} else {
-			GdaDataHandler *data_handler = gda_server_provider_get_data_handler_gtype
+			GdaDataHandler *data_handler = gda_server_provider_get_data_handler_g_type
 				(provider, cnc, G_VALUE_TYPE(value));
 			if (data_handler == NULL)
 				param_values[i] = NULL;
 			else
 				param_values[i] = gda_data_handler_get_str_from_value (data_handler,
 										       value);
-			/* g_print ("--- PV=%s\n", param_values[i]); */
+			//g_print ("--- PV=%s\n", param_values[i]);
 
 			mysql_bind_param[i].buffer_type = MYSQL_TYPE_STRING;
 			mysql_bind_param[i].buffer = g_strdup (param_values[i]);
 			mysql_bind_param[i].buffer_length = strlen (param_values[i]);
 			mysql_bind_param[i].length = g_malloc0 (sizeof(unsigned long));
-
 		}
-
 	}
 		
 	if (mysql_stmt_bind_param (cdata->mysql_stmt, mysql_bind_param)) {
@@ -1450,7 +1473,6 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 		return NULL;
 	}
 
-
 	/* use cursor when retrieving result */
 	if ((model_usage & GDA_STATEMENT_MODEL_RANDOM_ACCESS) == 0 &&
 	    gda_statement_get_statement_type (stmt) == GDA_SQL_STATEMENT_SELECT) {
@@ -1467,12 +1489,41 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 #endif
 	}
 
-	
 	/* add a connection event for the execution */
 	event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
         gda_connection_event_set_description (event, _GDA_PSTMT (ps)->sql);
         gda_connection_add_event (cnc, event);
 
+	if (empty_rs) {
+		/* There are some missing parameters, so the SQL can't be executed but we still want
+		 * to execute something to get the columns correctly. A possibility is to actually
+		 * execute another SQL which is the code shown here.
+		 *
+		 * To adapt depending on the C API and its features */
+		GdaStatement *stmt_for_empty;
+                gchar *sql_for_empty;
+                stmt_for_empty = gda_select_alter_select_for_empty (stmt, error);
+                if (!stmt_for_empty)
+                        return NULL;
+                sql_for_empty = gda_statement_to_sql (stmt_for_empty, NULL, error);
+                g_object_unref (stmt_for_empty);
+                if (!sql_for_empty)
+                        return NULL;
+
+		/* This is a re-prepare of the statement.  The function mysql_stmt_prepare
+		 * will handle this on the server side. */
+		if (mysql_stmt_prepare (cdata->mysql_stmt, sql_for_empty, strlen (sql_for_empty))) {
+			GdaConnectionEvent *event = _gda_mysql_make_error
+				(cdata->cnc, NULL, cdata->mysql_stmt, NULL);
+			return NULL;
+		}
+
+		/* Execute the 'sql_for_empty' SQL code */
+                g_free (sql_for_empty);
+
+		//TO_IMPLEMENT;
+	}
+
 	
 	GObject *return_value = NULL;
 	if (mysql_stmt_execute (cdata->mysql_stmt)) {
@@ -1505,7 +1556,6 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			// TO_IMPLEMENT;
 			/* Create a #GdaSet containing "IMPACTED_ROWS" */
 			/* Create GdaConnectionEvent notice with the type of command and impacted rows */
-
 			
 			affected_rows = mysql_stmt_affected_rows (cdata->mysql_stmt);
 			if (affected_rows >= 0) {
@@ -1523,12 +1573,9 @@ gda_mysql_provider_statement_execute (GdaServerProvider               *provider,
 			} else {
 				return_value = (GObject *) gda_data_model_array_new (0);
 			}
-			
 		}
-		
 	}
 	return return_value;
-	
 }
 
 /*
