@@ -22,10 +22,10 @@ typedef struct {
 } SigEvent;
 static GSList *signals = NULL;
 static void     monitor_model_signals (GdaDataModel *model);
-static void     signal_callback (GdaDataModel *model, gint row, gchar *type);
 static void     clear_signals (void);
 static gboolean check_expected_signal (GdaDataModel *model, gchar type, gint row);
 static gboolean check_no_expected_signal (GdaDataModel *model);
+static gboolean compare_data_models (GdaDataModel *model1, GdaDataModel *model2, GError **error);
 
 /* utility functions */
 static GdaConnection *setup_connection (void);
@@ -41,6 +41,7 @@ static gboolean check_set_values (GdaDataModel *model, gint row, GList *set_valu
 				  GdaConnection *cnc, GdaStatement *stmt, GdaSet *stmt_params);
 static gint check_append_values (GdaDataModel *model, GList *set_values,
 				 GdaConnection *cnc, GdaStatement *stmt, GdaSet *stmt_params);
+static void dump_data_model (GdaDataModel *model);
 
 typedef gboolean (*TestFunc) (GdaConnection *);
 static gint test1 (GdaConnection *cnc);
@@ -57,9 +58,12 @@ static gint test11 (GdaConnection *cnc);
 static gint test12 (GdaConnection *cnc);
 static gint test13 (GdaConnection *cnc);
 static gint test14 (GdaConnection *cnc);
+static gint test15 (GdaConnection *cnc);
+static gint test16 (GdaConnection *cnc);
+static gint test17 (GdaConnection *cnc);
 
 TestFunc tests[] = {
-        test1,
+	test1,
         test2,
         test3,
 	test4,
@@ -72,7 +76,10 @@ TestFunc tests[] = {
 	test11,
 	test12,
 	test13,
-	test14
+	test14,
+	test15,
+	test16,
+	test17
 };
 
 int
@@ -99,10 +106,9 @@ main (int argc, char **argv)
 	}
 
 	g_object_unref (cnc);
-	if (number_failed == 0)
-		g_print ("Ok.\n");
-	else
-		g_print ("%d failed\n", number_failed);
+	
+	g_print ("TESTS COUNT: %d\n", i);
+	g_print ("FAILURES: %d\n", number_failed);
 
 	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
@@ -1243,9 +1249,15 @@ test12 (GdaConnection *cnc)
 	clear_signals ();
 
 	/* create GdaDataSelect */
-	stmt = stmt_from_string ("SELECT * FROM customers WHERE id <= ##id::int");
+	stmt = stmt_from_string ("SELECT * FROM customers WHERE id <= ##id::int ORDER BY name");
 	g_assert (gda_statement_get_parameters (stmt, &params, NULL));
-	g_assert (gda_set_set_holder_value (params, &error, "id", "9"));
+	g_assert (gda_set_set_holder_value (params, &error, "id", 9));
+
+	model = gda_connection_statement_execute_select (cnc, stmt, params, &error);
+	g_assert (model);
+	gda_data_model_dump (model, stdout);
+	g_object_unref (model);
+
 	model = gda_connection_statement_execute_select_full (cnc, stmt, params, 
 							      GDA_STATEMENT_MODEL_CURSOR_FORWARD, 
 							      NULL, &error);
@@ -1308,6 +1320,47 @@ test12 (GdaConnection *cnc)
 #endif
 		goto out;
 	}
+	gda_value_free (value);
+
+	/**/
+	g_value_set_int ((value = gda_value_new (G_TYPE_INT)), 1);
+	if (! gda_data_model_iter_set_value_at (iter, 0, value, &error)) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+		g_print ("GdaDataModelIter value set failed: %s\n",
+			 error && error->message ? error->message : "No detail");
+#endif
+		goto out;
+	}
+	if (! check_expected_signal (model, 'U', 1)) {
+		nfailed++;
+		goto out;
+	}
+	cvalue = gda_data_model_iter_get_value_at (iter, 0);
+	if (!cvalue) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+		g_print ("gda_data_model_iter_get_value_at() failed after modification\n");
+#endif
+		goto out;
+	}
+	if (gda_value_differ (cvalue, value)) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+		g_print ("gda_data_model_iter_get_value_at() and modified value differ\n");
+#endif
+		goto out;
+	}
+
+	GdaDataModel *rerun;
+	rerun = gda_connection_statement_execute_select (cnc, stmt, params, &error);
+	g_assert (rerun);
+	gda_data_model_dump (rerun, stdout);
+	
+	gda_value_free (value);
+	
+	
+
 	g_object_unref (iter);
 
 
@@ -1506,6 +1559,326 @@ test14 (GdaConnection *cnc)
 	return nfailed;
 }
 
+/*
+ * - Create a GdaDataSelect with a parameter
+ * - Set modification statements
+ * - Refresh data model and compare with direct SELECT.
+ *
+ * Returns the number of failures 
+ */
+static gint
+test15 (GdaConnection *cnc)
+{
+	GError *error = NULL;
+        GdaDataModel *model, *rerun;
+        GdaStatement *stmt;
+        GdaSet *params;
+        gint nfailed = 0;
+
+	clear_signals ();
+
+        /* create GdaDataModelQuery */
+        stmt = stmt_from_string ("SELECT * FROM customers WHERE id <= ##theid::gint");
+        g_assert (gda_statement_get_parameters (stmt, &params, NULL));
+
+	if (! gda_set_set_holder_value (params, &error, "theid", 9)) {
+                nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Can't set 'theid' value: %s \n",
+                         error && error->message ? error->message : "No detail");
+#endif
+                goto out;
+        }
+        model = gda_connection_statement_execute_select (cnc, stmt, params, &error);
+        g_assert (model);
+
+	monitor_model_signals (model);
+
+	/**/
+	g_object_set_data (G_OBJECT (model), "mydata", "hey");
+
+	/**/
+	g_object_set (G_OBJECT (model), "auto-reset", TRUE, NULL);
+	if (! gda_set_set_holder_value (params, &error, "theid", 3)) {
+                nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Can't set 'theid' value: %s \n",
+                         error && error->message ? error->message : "No detail");
+#endif
+                goto out;
+        }
+	check_expected_signal (model, 'R', -1);
+
+	const gchar *dstr = g_object_get_data (G_OBJECT (model), "mydata");
+	if (!dstr || strcmp (dstr, "hey")) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Data model lost custom added data: expected 'hey' and got '%s'\n",
+			 dstr);
+#endif
+                goto out;
+	}
+
+	rerun = gda_connection_statement_execute_select (cnc, stmt, params, &error);
+	g_assert (rerun);
+	if (! compare_data_models (model, rerun, NULL)) {
+                nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Data model differs after a refresh\n");
+#endif
+                goto out;
+        }
+        g_object_unref (rerun);
+
+ out:
+        g_object_unref (model);
+        g_object_unref (stmt);
+
+        return nfailed;
+}
+
+/*
+ * - Create a GdaDataSelect with a missing parameter
+ * - Set modification statements
+ * - Refresh data model and compare with direct SELECT.
+ *
+ * Returns the number of failures 
+ */
+static gint
+test16 (GdaConnection *cnc)
+{
+	GError *error = NULL;
+        GdaDataModel *model, *rerun;
+        GdaStatement *stmt;
+        GdaSet *params;
+        gint nfailed = 0;
+
+	clear_signals ();
+
+        /* create GdaDataModelQuery */
+        stmt = stmt_from_string ("SELECT * FROM customers WHERE id <= ##theid::gint");
+        g_assert (gda_statement_get_parameters (stmt, &params, NULL));
+
+        model = gda_connection_statement_execute_select_full (cnc, stmt, params, GDA_STATEMENT_MODEL_ALLOW_NOPARAM,
+							      NULL, &error);
+	if (!model) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Can't run a SELECT with invalid parameters and the "
+			 "GDA_STATEMENT_MODEL_ALLOW_NOPARAM flag: %s\n",
+                         error && error->message ? error->message : "No detail");
+#endif
+                goto out;
+	}
+
+	if (gda_data_model_get_n_rows (model) != 0) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Data model should report 0 row and reports: %d",
+			 gda_data_model_get_n_rows (model));
+#endif
+                goto out;
+	}
+
+	dump_data_model (model);
+
+	/* set a custom data */
+	g_object_set_data (G_OBJECT (model), "mydata", "hey");
+	monitor_model_signals (model);
+
+	/**/
+	if (! gda_set_set_holder_value (params, &error, "theid", 9)) {
+                nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Can't set 'theid' value: %s \n",
+                         error && error->message ? error->message : "No detail");
+#endif
+                goto out;
+        }
+	check_expected_signal (model, 'R', -1);
+
+	const gchar *dstr = g_object_get_data (G_OBJECT (model), "mydata");
+	if (!dstr || strcmp (dstr, "hey")) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Data model lost custom added data: expected 'hey' and got '%s'\n",
+			 dstr);
+#endif
+                goto out;
+	}
+
+	dump_data_model (model);
+
+	/**/
+	if (! gda_set_set_holder_value (params, &error, "theid", 120)) {
+                nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Can't set 'theid' value: %s \n",
+                         error && error->message ? error->message : "No detail");
+#endif
+                goto out;
+        }
+	check_expected_signal (model, 'R', -1);
+
+	dstr = g_object_get_data (G_OBJECT (model), "mydata");
+	if (!dstr || strcmp (dstr, "hey")) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Data model lost custom added data: expected 'hey' and got '%s'\n",
+			 dstr);
+#endif
+                goto out;
+	}
+
+	dump_data_model (model);
+
+	rerun = gda_connection_statement_execute_select (cnc, stmt, params, &error);
+	g_assert (rerun);
+	if (! compare_data_models (model, rerun, NULL)) {
+                nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Data model differs after a refresh\n");
+#endif
+                goto out;
+        }
+        g_object_unref (rerun);
+
+	/**/
+	gda_holder_force_invalid (GDA_HOLDER (params->holders->data));
+	check_expected_signal (model, 'R', -1);
+	dump_data_model (model);
+
+ out:
+        g_object_unref (model);
+        g_object_unref (stmt);
+
+        return nfailed;
+}
+
+/*
+ * - Create a GdaDataSelect with a missing parameter
+ * - run it to be empty and create an iterator
+ * - set param's value so it refreshes
+ * - make sure the iterator's types are up to date with the data model ones.
+ *
+ * Returns the number of failures 
+ */
+static gint
+test17 (GdaConnection *cnc)
+{
+	GError *error = NULL;
+        GdaDataModel *model;
+        GdaStatement *stmt;
+	GdaDataModelIter *iter;
+        GdaSet *params;
+        gint nfailed = 0;
+
+        /* create GdaDataModelQuery */
+        stmt = stmt_from_string ("SELECT * FROM customers WHERE id <= ##theid::gint");
+        g_assert (gda_statement_get_parameters (stmt, &params, NULL));
+
+        model = gda_connection_statement_execute_select_full (cnc, stmt, params, GDA_STATEMENT_MODEL_ALLOW_NOPARAM,
+							      NULL, &error);
+	g_assert (model);
+	iter = gda_data_model_create_iter (model);
+	
+	gint i, nullcol = -1;
+	GdaColumn *column;
+	GSList *list;
+	for (i = 0, list = ((GdaSet*) iter)->holders;
+	     list;
+	     i++, list = list->next) {
+		if (gda_holder_get_g_type ((GdaHolder *) list->data) == GDA_TYPE_NULL) 
+			nullcol = i;
+		column = gda_data_model_describe_column (model, i);
+		if (gda_holder_get_g_type ((GdaHolder *) list->data) != 
+		    gda_column_get_g_type (column)) {
+			nfailed++;
+#ifdef CHECK_EXTRA_INFO
+			g_print ("Iter's GdaHolder for column %d reports the '%s' type when it should be '%s'",
+				 nullcol, 
+				 g_type_name (gda_holder_get_g_type ((GdaHolder *) list->data)),
+				 g_type_name (gda_column_get_g_type (column)));
+#endif
+			goto out;
+		}
+	}
+	if (nullcol == -1) {
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Could not find a GDA_TYPE_NULL column, test will be invalid");
+#endif
+		goto out;
+	}
+
+	/**/
+	if (! gda_set_set_holder_value (params, &error, "theid", 9)) {
+                nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Can't set 'theid' value: %s \n",
+                         error && error->message ? error->message : "No detail");
+#endif
+                goto out;
+        }
+
+	column = gda_data_model_describe_column (model, nullcol);
+	if (gda_holder_get_g_type ((GdaHolder *) g_slist_nth_data (((GdaSet*) iter)->holders, nullcol)) !=
+	    gda_column_get_g_type (column)) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Iter's GdaHolder for column %d reports the '%s' type when it should be '%s'",
+			 nullcol, 
+			 g_type_name (gda_holder_get_g_type ((GdaHolder *) g_slist_nth_data (((GdaSet*) iter)->holders, 
+											     nullcol))),
+			 g_type_name (gda_column_get_g_type (column)));
+#endif
+                goto out;
+	}
+
+	/**/
+	if (gda_data_model_iter_is_valid (iter)) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Iter should be invalid, and it is at row %d\n",
+			 gda_data_model_iter_get_row (iter));
+#endif
+                goto out;
+	}
+	dump_data_model (model);
+	while (gda_data_model_iter_move_next (iter));
+	if (gda_data_model_iter_is_valid (iter)) {
+		nfailed++;
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Iter could not be moved up to the end, and remained 'locked' at row %d\n",
+			 gda_data_model_iter_get_row (iter));
+#endif
+                goto out;
+	}
+
+
+ out:
+        g_object_unref (model);
+        g_object_unref (stmt);
+
+        return nfailed;
+}
+
+
+static void
+dump_data_model (GdaDataModel *model)
+{
+	gint i, ncols;
+	g_print ("=== Data Model Dump ===\n");
+	ncols = gda_data_model_get_n_columns (model);
+	for (i = 0; i < ncols; i++) {
+		GdaColumn *col;
+		col = gda_data_model_describe_column (model, i);
+		if (!col)
+			g_print ("Missing column %d\n", i);
+		g_print ("Column %d: ptr=>%p type=>%s\n", i, col, g_type_name (gda_column_get_g_type (col)));
+	}
+	gda_data_model_dump (model, stdout);
+}
+
 /*
  * Checking value function:
  *  - reads the value of @model at the provided column and row and compares with the expected @set_value
@@ -1778,14 +2151,6 @@ check_append_values (GdaDataModel *model, GList *set_values, GdaConnection *cnc,
 /* 
  * signals checking 
  */
-static void
-monitor_model_signals (GdaDataModel *model)
-{
-	g_signal_connect (model, "row-updated", G_CALLBACK (signal_callback), "U");
-	g_signal_connect (model, "row-removed", G_CALLBACK (signal_callback), "D");
-	g_signal_connect (model, "row-inserted", G_CALLBACK (signal_callback), "I");
-}
-
 static void
 signal_callback (GdaDataModel *model, gint row, gchar *type)
 {
@@ -1802,6 +2167,32 @@ signal_callback (GdaDataModel *model, gint row, gchar *type)
 #endif
 }
 
+static void
+signal_callback_1 (GdaDataModel *model, gchar *type)
+{
+	SigEvent *se;
+	se = g_new0 (SigEvent, 1);
+	se->model = model;
+	se->type = *type;
+	se->row = -1;
+	signals = g_slist_append (signals, se);
+#ifdef CHECK_EXTRA_INFO
+	g_print ("Received '%s' signal from model %p\n",
+		 *type == 'R' ? "reset" : "???",
+		 model);
+#endif
+}
+
+static void
+monitor_model_signals (GdaDataModel *model)
+{
+	g_signal_connect (model, "row-updated", G_CALLBACK (signal_callback), "U");
+	g_signal_connect (model, "row-removed", G_CALLBACK (signal_callback), "D");
+	g_signal_connect (model, "row-inserted", G_CALLBACK (signal_callback), "I");
+	g_signal_connect (model, "reset", G_CALLBACK (signal_callback_1), "R");
+}
+
+
 static void
 clear_signals (void)
 {
@@ -1829,13 +2220,48 @@ check_expected_signal (GdaDataModel *model, gchar type, gint row)
 	}
 #ifdef CHECK_EXTRA_INFO
 	else {
-		g_print ("Expected signal '%s' for row %d from model %p and got ",
-			 (type == 'I') ? "row-inserted" : ((type == 'D') ? "row-removed" : "row-updated"),
+		gchar *exp;
+		switch (type) {
+		case 'I':
+			exp = "row-inserted";
+			break;
+		case 'U':
+			exp = "row-updated";
+			break;
+		case 'D':
+			exp = "row-removed";
+			break;
+		case 'R':
+			exp = "reset";
+			break;
+		default:
+			exp = "???";
+			break;
+		}
+		g_print ("Expected signal '%s' for row %d from model %p and got ", exp,
 			 row, model);
-		if (se)
+		if (se) {
+			switch (se->type) {
+			case 'I':
+				exp = "row-inserted";
+				break;
+			case 'U':
+				exp = "row-updated";
+				break;
+			case 'D':
+				exp = "row-removed";
+				break;
+			case 'R':
+				exp = "reset";
+				break;
+			default:
+				exp = "???";
+				break;
+			}
 			g_print ("signal '%s' for row %d from model %p\n",
-				 (se->type == 'I') ? "row-inserted" : ((se->type == 'D') ? "row-removed" : "row-updated"),
+				 exp,
 				 se->row, se->model);
+		}
 		else
 			g_print ("no signal\n");
 	}
@@ -1865,3 +2291,41 @@ check_no_expected_signal (GdaDataModel *model)
 
 	return TRUE;
 }
+
+static gboolean
+compare_data_models (GdaDataModel *model1, GdaDataModel *model2, GError **error)
+{
+        GdaDataComparator *cmp;
+        GError *lerror = NULL;
+        cmp = (GdaDataComparator*) gda_data_comparator_new (model1, model2);
+        if (! gda_data_comparator_compute_diff (cmp, &lerror)) {
+#ifdef CHECK_EXTRA_INFO
+                g_print ("Could not compute the data model differences: %s\n",
+                         lerror && lerror->message ? lerror->message : "No detail");
+		g_print ("Model1 is:\n");
+                gda_data_model_dump (model1, stdout);
+                g_print ("Model2 is:\n");
+                gda_data_model_dump (model2, stdout);
+#endif
+                goto onerror;
+        }
+        if (gda_data_comparator_get_n_diffs (cmp) != 0) {
+#ifdef CHECK_EXTRA_INFO
+                g_print ("There are some differences when comparing data models...\n");
+                g_print ("Model1 is:\n");
+                gda_data_model_dump (model1, stdout);
+                g_print ("Model2 is:\n");
+                gda_data_model_dump (model2, stdout);
+#endif
+		g_set_error (&lerror, 0, 0,
+			     "There are some differences when comparing data models...");
+                goto onerror;
+        }
+        g_object_unref (cmp);
+
+        return TRUE;
+
+ onerror:
+        g_propagate_error (error, lerror);
+        return FALSE;
+}
