@@ -26,10 +26,84 @@
 #ifdef HAVE_HISTORY
 #include <readline/history.h>
 #endif
-
+#include <sql-parser/gda-statement-struct-util.h>
 
 /*
- * command_exec_result_free
+ *  gda_internal_command_arg_remove_quotes
+ *
+ * If @str has simple or double quotes, remove those quotes, otherwise does nothing.
+ *
+ * Returns: @str
+ */
+gchar *
+gda_internal_command_arg_remove_quotes (gchar *str)
+{
+	glong total;
+        gchar *ptr;
+        glong offset = 0;
+        char delim;
+
+        if (!str)
+                return NULL;
+        delim = *str;
+        if ((delim != '\'') && (delim != '"'))
+                return str;
+
+
+        total = strlen (str);
+        if (str[total-1] == delim) {
+                /* string is correclty terminated by a double quote */
+                g_memmove (str, str+1, total-2);
+                total -=2;
+        }
+        else {
+                /* string is _not_ correclty terminated by a double quote */
+                g_memmove (str, str+1, total-1);
+                total -=1;
+        }
+        str[total] = 0;
+
+        ptr = (gchar *) str;
+        while (offset < total) {
+                /* we accept the "''" as a synonym of "\'" */
+                if (*ptr == delim) {
+                        if (*(ptr+1) == delim) {
+                                g_memmove (ptr+1, ptr+2, total - offset);
+                                offset += 2;
+                        }
+                        else {
+                                *str = 0;
+                                return str;
+                        }
+                }
+                if (*ptr == '\\') {
+                        if (*(ptr+1) == '\\') {
+                                g_memmove (ptr+1, ptr+2, total - offset);
+                                offset += 2;
+                        }
+                        else {
+                                if (*(ptr+1) == delim) {
+                                        *ptr = delim;
+                                        g_memmove (ptr+1, ptr+2, total - offset);
+                                        offset += 2;
+                                }
+                                else {
+                                        *str = 0;
+                                        return str;
+                                }
+			}
+                }
+                else
+                        offset ++;
+
+                ptr++;
+        }
+
+        return str;
+}
+
+/*
+ * gda_internal_command_exec_result_free
  *
  * Clears the memory associated with @res
  */
@@ -303,6 +377,11 @@ gda_internal_command_execute (GdaInternalCommandsList *commands_list,
 		args = command->arguments_delimiter_func (command_str);
 	else
 		args = default_gda_internal_commandargs_func (command_str);
+	if (command->unquote_args) {
+		gint i;
+		for (i = 1; args[i]; i++) 
+			gda_internal_command_arg_remove_quotes (args[i]);
+	}
 	res = command->command_func (cnc, (const gchar **) &(args[1]), 
 				     error, command->user_data);
 	
@@ -433,7 +512,8 @@ gda_internal_command_list_tables (GdaConnection *cnc, const gchar **args, GError
 			"table_type LIKE '%TABLE%' AND table_short_name = table_name "
 			"ORDER BY table_schema, table_name";
 
-		g_value_set_string (v = gda_value_new (G_TYPE_STRING), args[0]);
+		gchar *tmp = gda_sql_identifier_remove_quotes (g_strdup (args[0]));
+		g_value_take_string (v = gda_value_new (G_TYPE_STRING), tmp);
 		model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error, "tname", v, NULL);
 		gda_value_free (v);
 	}
@@ -538,67 +618,6 @@ gda_internal_command_list_schemas (GdaConnection *cnc, const gchar **args, GErro
 	return res;
 }
 
-GdaInternalCommandResult *
-gda_internal_command_list_queries (GdaConnection *cnc, const gchar **args,
-				   GError **error, gpointer data)
-{
-	GdaInternalCommandResult *res = NULL;
-	GdaDataModel *model;
-	const gchar *qname = NULL;
-	gboolean with_sql_def = FALSE;
-
-	if (!cnc) {
-		g_set_error (error, 0, 0, _("No current connection"));
-		return NULL;
-	}
-
-	if (args[0] && *args[0]) {
-		if (!strcmp (args[0], "+"))
-			with_sql_def = TRUE;
-		else {
-			qname = args[0];
-			if (args[1] && (*args[1] == '+'))
-				with_sql_def = TRUE;
-		}
-	}
-
-	if (with_sql_def)
-		model = gda_data_model_array_new_with_g_types (5,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING);
-	else
-		model = gda_data_model_array_new_with_g_types (4,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING,
-							       G_TYPE_STRING);
-	gda_data_model_set_column_title (model, 0, _("Name"));
-	gda_data_model_set_column_title (model, 1, _("Type"));
-	gda_data_model_set_column_title (model, 2, _("Description"));
-	gda_data_model_set_column_title (model, 3, _("Parameters"));
-	if (with_sql_def)
-		gda_data_model_set_column_title (model, 4, _("SQL"));
-	g_object_set_data (G_OBJECT (model), "name", _("List of queries"));
-
-	if (qname) {
-		TO_IMPLEMENT;
-	}
-	else {
-		
-
-		TO_IMPLEMENT;
-	}
-
-	res = g_new0 (GdaInternalCommandResult, 1);
-	res->type = GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL;
-	res->u.model = model;
-
-	return res;
-}
-
 GdaMetaStruct *
 gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args, GError **error)
 {
@@ -634,19 +653,21 @@ gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args,
 		}
 		else {
 			/* try to find it as a table or view */
-			if (!gda_meta_struct_complement (mstruct, GDA_META_DB_UNKNOWN, NULL, NULL, v, NULL)) {
-				if (g_str_has_suffix (arg, "=") && (*arg != '=')) {
-					GdaMetaDbObject *dbo;
-					gchar *str;
-					str = g_strdup (arg);
-					str[strlen (str) - 1] = 0;
-					g_value_take_string (v, str);
-					dbo = gda_meta_struct_complement (mstruct, GDA_META_DB_UNKNOWN, 
-									  NULL, NULL, v, NULL);
-					if (dbo)
-						gda_meta_struct_complement_depend (mstruct, dbo, NULL);
-				}
+			if (g_str_has_suffix (arg, "=") && (*arg != '=')) {
+				GdaMetaDbObject *dbo;
+				gchar *str;
+				str = g_strdup (arg);
+				str[strlen (str) - 1] = 0;
+				g_value_take_string (v, str);
+				dbo = gda_meta_struct_complement (mstruct, GDA_META_DB_UNKNOWN, 
+								  NULL, NULL, v, error);
+				if (dbo)
+					gda_meta_struct_complement_depend (mstruct, dbo, error);
+				else
+					goto onerror;
 			}
+			else if (!gda_meta_struct_complement (mstruct, GDA_META_DB_UNKNOWN, NULL, NULL, v, error))
+				goto onerror;
 		}
 	}
 
@@ -665,6 +686,21 @@ gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args,
 	return NULL;
 }
 
+static void
+meta_table_column_foreach_attribute_func (const gchar *att_name, const GValue *value, GString **string)
+{
+	if (!strcmp (att_name, GDA_ATTRIBUTE_AUTO_INCREMENT) && 
+	    (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN) && 
+	    g_value_get_boolean (value)) {
+		if (*string) {
+			g_string_append (*string, ", ");
+			g_string_append (*string, _("Auto increment"));
+		}
+		else
+			*string = g_string_new (_("Auto increment"));
+	}
+}
+
 GdaInternalCommandResult *
 gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			     GError **error, gpointer data)
@@ -800,7 +836,8 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			GdaMetaTableColumn *tcol = GDA_META_TABLE_COLUMN (list->data);
 			GList *values = NULL;
 			GValue *val;
-			
+			GString *string = NULL;
+
 			g_value_set_string ((val = gda_value_new (G_TYPE_STRING)), tcol->column_name);
 			values = g_list_append (values, val);
 			g_value_set_string ((val = gda_value_new (G_TYPE_STRING)), tcol->column_type);
@@ -809,28 +846,10 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 			values = g_list_append (values, val);
 			g_value_set_string ((val = gda_value_new (G_TYPE_STRING)), tcol->default_value);
 			values = g_list_append (values, val);
-			if (tcol->extra) {
-				GString *string;
-				gint i;
-				gboolean first = TRUE;
-				string = g_string_new ("");
-				for (i = 0; i < tcol->extra->len; i++) {
-					const gchar *tmp = g_array_index (tcol->extra, gchar *, i);
-					const gchar *astmp = NULL;
-					if (!strcmp (tmp, GDA_EXTRA_AUTO_INCREMENT)) 
-						astmp = _("Auto increment");
-					else {
-						g_warning ("Unknown extra keyword '%s'", tmp);
-						TO_IMPLEMENT;
-					}
-					if (astmp) {
-						if (first)
-							first = FALSE;
-						else
-							g_string_append (string, ", ");
-						g_string_append (string, astmp);
-					}
-				}
+
+			gda_meta_table_column_foreach_attribute (tcol, 
+					      (GdaAttributesManagerFunc) meta_table_column_foreach_attribute_func, &string);
+			if (string) {
 				g_value_take_string ((val = gda_value_new (G_TYPE_STRING)), string->str);
 				g_string_free (string, FALSE);
 			}
@@ -993,7 +1012,7 @@ gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 							for (j = 0; j < cnrows; j++) {
 								if (j > 0)
 									g_string_append (string, ", ");
-								cvalue = gda_data_model_get_value_at (cols, 2, j, error);
+								cvalue = gda_data_model_get_value_at (cols, 3, j, error);
 								if (!cvalue) {
 									gda_internal_command_exec_result_free (res);
 									res = NULL;
