@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007 The GNOME Foundation.
+ * Copyright (C) 2007 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -32,7 +32,7 @@
 typedef enum {
 	GDA_INTERNAL_COMMAND_RESULT_EMPTY,
 	GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL,
-	GDA_INTERNAL_COMMAND_RESULT_PLIST,
+	GDA_INTERNAL_COMMAND_RESULT_SET,
 	GDA_INTERNAL_COMMAND_RESULT_TXT,
 	GDA_INTERNAL_COMMAND_RESULT_TXT_STDOUT,
 	GDA_INTERNAL_COMMAND_RESULT_MULTIPLE,
@@ -43,7 +43,7 @@ typedef struct {
 	GdaInternalCommandResultType type;
 	union {
 		GdaDataModel     *model;
-		GdaParameterList *plist;
+		GdaSet           *set;
 		GString          *txt;
 		GSList           *multiple_results; /* for GDA_INTERNAL_COMMAND_RESULT_MULTIPLE */
 	} u;
@@ -52,9 +52,7 @@ typedef struct {
 /*
  * Command definition
  */
-typedef GdaInternalCommandResult *(*GdaInternalCommandFunc) (GdaConnection *, GdaDict *,
-							     const gchar **,
-							     GError **, gpointer);
+typedef GdaInternalCommandResult *(*GdaInternalCommandFunc) (GdaConnection *, const gchar **, GError **, gpointer);
 typedef gchar                   **(*GdaInternalCommandArgsFunc) (const gchar *);
 typedef struct {
 	gchar    *name;
@@ -69,6 +67,7 @@ typedef struct {
 	GdaInternalCommandFunc     command_func;
 	gpointer                   user_data;
 	GdaInternalCommandArgsFunc arguments_delimiter_func;
+	gboolean                   unquote_args;
 } GdaInternalCommand;
 
 typedef struct {
@@ -79,34 +78,30 @@ typedef struct {
 	GSList    *group_ordered;
 } GdaInternalCommandsList;
 
+gchar                    *gda_internal_command_arg_remove_quotes (gchar *str);
+
 /* Commands execution */
 GdaInternalCommandResult *gda_internal_command_execute (GdaInternalCommandsList *commands_list,
-							GdaConnection *cnc, GdaDict *dict, 
-							const gchar *command_str, GError **error);
+							GdaConnection *cnc, const gchar *command_str, GError **error);
 void                      gda_internal_command_exec_result_free (GdaInternalCommandResult *res);
 
-
 /* Available commands */
-GdaInternalCommandResult *gda_internal_command_help (GdaConnection *cnc, GdaDict *dict, 
-						     const gchar **args,
+GdaInternalCommandResult *gda_internal_command_help (GdaConnection *cnc, const gchar **args,
 						     GError **error, GdaInternalCommandsList *clist);
-GdaInternalCommandResult *gda_internal_command_history (GdaConnection *cnc, GdaDict *dict, 
-							const gchar **args,
+GdaInternalCommandResult *gda_internal_command_history (GdaConnection *cnc, const gchar **args,
 							GError **error, gpointer data);
-GdaInternalCommandResult *gda_internal_command_dict_sync (GdaConnection *cnc, GdaDict *dict,
-							  const gchar **args,
+GdaInternalCommandResult *gda_internal_command_dict_sync (GdaConnection *cnc, const gchar **args,
 							  GError **error, gpointer data);
-GdaInternalCommandResult *gda_internal_command_dict_save (GdaConnection *cnc, GdaDict *dict, 
-							  const gchar **args,
-							  GError **error, gpointer data);
-GdaInternalCommandResult *gda_internal_command_list_tables_views (GdaConnection *cnc, GdaDict *dict, 
-								  const gchar **args,
-								  GError **error, gpointer data);
-GdaInternalCommandResult *gda_internal_command_list_queries (GdaConnection *cnc, GdaDict *dict, 
-							     const gchar **args,
+GdaInternalCommandResult *gda_internal_command_list_tables (GdaConnection *cnc, const gchar **args,
+							    GError **error, gpointer data);
+GdaInternalCommandResult *gda_internal_command_list_views (GdaConnection *cnc, const gchar **args,
+							   GError **error, gpointer data);
+GdaInternalCommandResult *gda_internal_command_list_schemas (GdaConnection *cnc, const gchar **args,
 							     GError **error, gpointer data);
-GdaInternalCommandResult *gda_internal_command_detail (GdaConnection *cnc, GdaDict *dict, 
-						       const gchar **args,
+GdaInternalCommandResult *gda_internal_command_detail (GdaConnection *cnc, const gchar **args,
 						       GError **error, gpointer data);
 
+/* Misc */
+GdaMetaStruct            *gda_internal_command_build_meta_struct (GdaConnection *cnc, const gchar **args, GError **error);
+
 #endif
